    MODULE accumulate_mod
!----------------------------------------------------------------------
! Routines to zero and accumulate sums and sums of squares.
! Also to convert to standard deviation
!----------------------------------------------------------------------
! .. Default Accessibility ..
      PUBLIC
! ..
! .. Private Statements ..
      PRIVATE :: dpkind, zero
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
      REAL (dpkind), PARAMETER :: zero = 0.0_dpkind
! ..
! .. Intrinsic Functions ..
      INTRINSIC KIND
! ..
    CONTAINS

!*********************************************************************

      ELEMENTAL SUBROUTINE accumulate(term,esum,sum_squares)
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (INOUT) :: esum, sum_squares
        REAL (dpkind), INTENT (IN) :: term
! ..
        esum = esum + term

        sum_squares = sum_squares + term*term

        RETURN

      END SUBROUTINE accumulate

!*********************************************************************

      ELEMENTAL SUBROUTINE stats_from_accum(esum,sum_squares,n,av,var,sd)
! .. Scalar Arguments ..
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: av, sd, var
        REAL (dpkind), INTENT (IN) :: esum, sum_squares
        INTEGER, INTENT (IN) :: n
! ..
! .. Local Scalars ..
        REAL (dpkind) :: local_av, local_sd, local_var, xn
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT, REAL, SQRT
! ..
        xn = REAL(n,kind=dpkind)

        local_av = esum/xn

        IF (PRESENT(av)) av = local_av

        IF (PRESENT(var) .OR. PRESENT(sd)) THEN

          local_var = sum_squares/xn - (esum/xn)**2

          local_sd = SQRT(local_var)

        END IF

        IF (PRESENT(var)) var = local_var

        IF (PRESENT(sd)) sd = local_sd

        RETURN

      END SUBROUTINE stats_from_accum

!*********************************************************************

    END MODULE accumulate_mod
