    MODULE cdf_aux_mod
! .. Implicit None Statement ..
      IMPLICIT NONE
! ..
! .. Parameters ..
      INTEGER, PARAMETER, PRIVATE :: dpkind = KIND(0.0D0)
      REAL (dpkind), PARAMETER, PRIVATE :: big = 1.0E10_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: half = 0.5_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: inf = 1.0E100_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: one = 1.0_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: small = 1.0E-10_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: small_prob = 1.0E-8_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: ten4 = 1.0E4_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: three = 3.0_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: zero = 0.0_dpkind
      REAL (dpkind), PARAMETER, PRIVATE :: large_prob = one - small_prob
! ..
! .. Generic Interface Blocks ..
      INTERFACE in_range
        MODULE PROCEDURE dbl_in_range, int_in_range
      END INTERFACE
! ..
! .. Intrinsic Functions ..
! .. Local Structures
! Each parameter has this associated fields:
!   * name
!   * no_check: for this value of 'which'
!   * low_bound: minimum value for the parameter
!   * high_bound: high value for the parameter
      INTRINSIC KIND
! ..
! .. Derived Type Declarations ..
      TYPE :: one_parameter
        CHARACTER (31) :: name
        INTEGER :: no_check
        REAL (dpkind) :: low_bound
        REAL (dpkind) :: high_bound
      END TYPE one_parameter
      TYPE :: the_distribution
        CHARACTER (31) :: name
        CHARACTER (80) :: line_1
        CHARACTER (80) :: line_2
        INTEGER :: max_which
        INTEGER :: nparam
        TYPE (one_parameter) :: parameters(6)
      END TYPE the_distribution
! ..
! .. Dependents ..
      TYPE (the_distribution), PARAMETER :: the_beta = the_distribution( &
        'cdf_beta', &
        '         Beta(X,1-X,A,B) = C,1-C     (Incomplete Beta)', &
        ' Input X or 1-X, A, B, and C or 1-C for beta distribution.',4,6, &
        (/one_parameter('cum',1,small_prob,large_prob), &
        one_parameter('ccum',1,zero,one),one_parameter('x',2,zero,one), &
        one_parameter('cx',2,zero,one),one_parameter('a',3,zero,inf), &
        one_parameter('b',4,zero,inf)/))
      TYPE (the_distribution), PARAMETER :: the_binomial = &
        the_distribution('cdf_binomial', &
        '         Bin(S,N,P,1-P) = C,1-C      (Binomial)', &
        ' Input S, N, P or 1-P, and C or 1-C for binomial distribution.', &
        4,6,(/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,large_prob), &
        one_parameter('s',2,zero,inf),one_parameter('n',3,one,big), &
        one_parameter('pr',4,zero,one),one_parameter('cpr',4,zero,one)/))
      TYPE (the_distribution), PARAMETER :: the_chi_square = &
        the_distribution('cdf_chi_square', &
        '         Chi(X,DF) = C,1-C           (Chi-Square)', &
        'Input X, DF, and C or 1-C for chi-square distribution.',3,4, &
        (/one_parameter('cum',1,small_prob,large_prob), &
        one_parameter('ccum',1,zero,one),one_parameter('x',2,zero,inf), &
        one_parameter('df',3,1.0E-100_dpkind,inf), &
        one_parameter(' ',0,zero,zero),one_parameter(' ',0,zero,zero)/))
      TYPE (the_distribution), PARAMETER :: the_dummy_binomial = &
        the_distribution('cdf_binomial',' ',' ',0,2, &
        (/one_parameter('pr',0,zero,half),one_parameter('cpr',0,zero,half &
        ),one_parameter(' ',0,zero,zero),one_parameter(' ',0,zero,zero), &
        one_parameter(' ',0,zero,zero),one_parameter(' ',0,zero,zero)/))
      TYPE (the_distribution), PARAMETER :: the_f = the_distribution( &
        'cdf_f','         F(X,DFN,DFD) = C,1-C        (F)', &
        ' Input X, DFN, DFD, and C or 1-C for F distribution.',4,5, &
        (/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,large_prob), &
        one_parameter('f',2,zero,inf),one_parameter('dfn',3,zero,inf), &
        one_parameter('dfd',4,zero,inf),one_parameter(' ',0,zero,zero)/))
      TYPE (the_distribution), PARAMETER :: the_gamma = the_distribution( &
        'cdf_gamma', &
        '        Gamma(X,A,R) = C,1-C        (Incomplete Gamma)', &
        ' Input X, A, R, and C or 1-C for gamma distribution.',4,5, &
        (/one_parameter('cum',1,small_prob,large_prob), &
        one_parameter('ccum',1,zero,one),one_parameter('x',2,zero,inf), &
        one_parameter('shape',3,zero,inf),one_parameter('scale',4,zero, &
        inf),one_parameter(' ',0,zero,zero)/))
      TYPE (the_distribution), PARAMETER :: the_negative_binomial = &
        the_distribution('cdf_negative_binomial','         &
        &        Negbin(X,S,P,1-P) = C,1-C   (Negative Binomial)',' &
        &Input X, S, P or 1-P, and C or 1-C for negative binomial &
        &distribution.',4,6,(/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,large_prob), &
        one_parameter('s',2,zero,inf),one_parameter('xn',3,one,inf), &
        one_parameter('pr',4,zero,one),one_parameter('cpr',4,zero,one)/))
      TYPE (the_distribution), PARAMETER :: the_non_central_chi_square = &
        the_distribution('cdf_nc_chisq', &
        '         Chn(X,DF,PNONC) = C,1-C     (Non-Central Chi-Square)',' &
        &Input X, DF, PNONC, and C or 1-C for non-central chisq &
        &distribution.',4,5,(/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,large_prob), &
        one_parameter('x',2,zero,inf),one_parameter('df',3,zero,inf), &
        one_parameter('pnonc',4,zero,ten4),one_parameter(' ',0,zero,zero) &
        /))
      TYPE (the_distribution), PARAMETER :: the_non_central_f = &
        the_distribution('cdf_nc_f', &
        '      Fn(X,DFN,DFD,PNONC) = C,1-C (Non-Central F)',' &
        &Input X, DFN, DFD, PNONC, and C or 1-C for non-central &
        &F distribution.',5,6,(/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,large_prob), &
        one_parameter('f',2,zero,inf),one_parameter('dfn',3,zero,inf), &
        one_parameter('dfd',4,zero,inf),one_parameter('pnonc',5,zero,ten4 &
        )/))
      TYPE (the_distribution), PARAMETER :: the_non_central_t = &
        the_distribution('cdf_non_central_t', &
        '         Tnc(TVAL,DF,PNONC) = C,1-C    (Non-Central T)', &
        ' Input TVAL, DF, PNONC, and C or 1-C for non-central t.',4,5, &
        (/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,large_prob), &
        one_parameter('t',2,-inf,inf),one_parameter('df',3,small,ten4), &
        one_parameter('pnonc',4,-ten4,ten4),one_parameter(' ',0,zero,zero &
        )/))
      TYPE (the_distribution), PARAMETER :: the_normal = the_distribution &
        ('cdf_normal','         N(X,MEAN,SD) = C,1-C        (Normal)', &
        ' Input X, MEAN, SD, and C or 1-C for normal distribution.',4,5, &
        (/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,one),one_parameter('x',2,-inf,inf), &
        one_parameter('mean',3,-inf,inf),one_parameter('sd',4,zero,inf), &
        one_parameter(' ',0,zero,zero)/))
      TYPE (the_distribution), PARAMETER :: the_poisson = &
        the_distribution('cdf_poisson', &
        '         P(N,MEAN) = C,1-C           (Poisson)', &
        ' Input N, MEAN, and C or 1-C for Poisson distribution.',3,4, &
        (/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,large_prob), &
        one_parameter('s',2,zero,inf),one_parameter('lambda',3,zero,inf), &
        one_parameter(' ',0,zero,zero),one_parameter(' ',0,zero,zero)/))
      TYPE (the_distribution), PARAMETER :: the_t = the_distribution( &
        'cdf_t','         T(TVAL,DF) = C,1-C          (T)', &
        ' Input TVAL, DF, and C or 1-C for t distribution.',3,4, &
        (/one_parameter('cum',1,zero,large_prob), &
        one_parameter('ccum',1,zero,large_prob), &
        one_parameter('t',2,-inf,inf),one_parameter('df',3,small,big), &
        one_parameter(' ',0,zero,zero),one_parameter(' ',0,zero,zero)/))
! ..
    CONTAINS

!*********************************************************************

      FUNCTION add_to_one(x,y,routine_name,x_name,y_name,bad_status, &
          status)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (IN) :: x, y
        INTEGER, INTENT (IN) :: bad_status
        INTEGER, OPTIONAL, INTENT (OUT) :: status
        CHARACTER (*), INTENT (IN) :: routine_name, x_name, y_name
! ..
! .. Function Return Value ..
        LOGICAL :: add_to_one
! ..
! .. Intrinsic Functions ..
        INTRINSIC ABS, EPSILON, PRESENT
! ..
! .. Executable Statements ..

        IF (ABS(((x+y)-half)-half)>three*EPSILON(one)) THEN
          add_to_one = .FALSE.
          IF (PRESENT(status)) THEN
            status = bad_status
          ELSE
            WRITE (*,90000) routine_name, x_name, y_name, x_name, x, &
              y_name, y
90000       FORMAT (' Error in routine: ',A/A,' + ',A, &
              ' must add to one'/'Value of ',A,' is: ',G15.8/'Value of ', &
              A,' is: ',G15.8/'Program aborting'/)
            STOP 'Error in cdf routine -- see output'
          END IF
        ELSE
          add_to_one = .TRUE.
        END IF

      END FUNCTION add_to_one

!*********************************************************************

      FUNCTION alngam(x)
!**********************************************************************
!     DOUBLE PRECISION FUNCTION ALNGAM(X)
!                 double precision LN of the GAMma function
!                              Function
!     Returns the natural logarithm of GAMMA(X).
!                              Arguments
!     X --> value at which scaled log gamma is to be returned
!                    X is DOUBLE PRECISION
!                              Method
!     If X .le. 6.0, then use recursion to get X below 3
!     then apply rational approximation number 5236 of
!     Hart et al, Computer Approximations, John Wiley and Sons, NY, 1968.
!     If X .gt. 6.0, then use recursion to get X to at least 12 and
!     then use formula 5423 of the same source.
!**********************************************************************
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Parameters ..
        REAL (dpkind), PARAMETER :: coef(5) = (/ &
          0.83333333333333023564E-1_dpkind, - &
          0.27777777768818808E-2_dpkind, 0.79365006754279E-3_dpkind, &
          -0.594997310889E-3_dpkind, 0.8065880899E-3_dpkind/)
        REAL (dpkind), PARAMETER :: scoefd(4) = (/ &
          0.62003838007126989331E2_dpkind, 0.9822521104713994894E1_dpkind &
          , -0.8906016659497461257E1_dpkind, &
          0.1000000000000000000E1_dpkind/)
        REAL (dpkind), PARAMETER :: scoefn(9) = (/ &
          0.62003838007127258804E2_dpkind, &
          0.36036772530024836321E2_dpkind, &
          0.20782472531792126786E2_dpkind, 0.6338067999387272343E1_dpkind &
          , 0.215994312846059073E1_dpkind, 0.3980671310203570498E0_dpkind &
          , 0.1093115956710439502E0_dpkind, 0.92381945590275995E-2_dpkind &
          , 0.29737866448101651E-2_dpkind/)
        REAL (dpkind), PARAMETER :: hln2pi = &
          0.91893853320467274178E0_dpkind
        REAL (dpkind), PARAMETER :: two = 2.0_dpkind
! ..
! .. Function Return Value ..
        REAL (dpkind) :: alngam
! ..
! .. Scalar Arguments ..
        REAL (dpkind) :: x
! ..
! .. Local Scalars ..
        REAL (dpkind) :: offset, prod, xx
        INTEGER :: i, n
! ..
! .. Intrinsic Functions ..
        INTRINSIC INT, LOG, REAL
! ..
! .. Executable Statements ..

        IF (x<=6.0E0_dpkind) THEN
          prod = one
          xx = x
          IF (x>3.0E0_dpkind) THEN
10          CONTINUE
            IF ( .NOT. xx>3.0E0_dpkind) GO TO 20
            xx = xx - one
            prod = prod*xx
            GO TO 10

20          CONTINUE
          END IF

          IF (x<two) THEN
30          CONTINUE
            IF ( .NOT. xx<two) GO TO 40
            prod = prod/xx
            xx = xx + one
            GO TO 30

40          CONTINUE
          END IF

          alngam = evaluate_polynomial(scoefn,xx-two)/ &
            evaluate_polynomial(scoefd,xx-two)

!     COMPUTE RATIONAL APPROXIMATION TO GAMMA(X)

          alngam = alngam*prod
          alngam = LOG(alngam)
        ELSE

          offset = hln2pi

!     IF NECESSARY MAKE X AT LEAST 12 AND CARRY CORRECTION IN OFFSET

          n = INT(12.0E0_dpkind-x)

          IF (n>0) THEN
            prod = one
            DO i = 1, n
              prod = prod*(x+REAL(i-1,kind=dpkind))
            END DO

            offset = offset - LOG(prod)
            xx = x + REAL(n,kind=dpkind)
          ELSE

            xx = x

!     COMPUTE POWER SERIES

          END IF

          alngam = evaluate_polynomial(coef,one/xx**2)/xx
          alngam = alngam + offset + (xx-half)*LOG(xx) - xx
        END IF

      END FUNCTION alngam

!*********************************************************************

      SUBROUTINE cdf_set_zero_finder(distrib,which,local)
! Called before solving a problem.
! Sets the bounds for the parameter to be found.
! The bounds are in the structure distrib, corresponding to the
! "which-th" parameter
! .. Use Statements ..
        USE zero_finder
! ..
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Parameters ..
        REAL (dpkind), PARAMETER :: absstp = half
        REAL (dpkind), PARAMETER :: atol = 1.0E-50_dpkind
        REAL (dpkind), PARAMETER :: relstp = half
        REAL (dpkind), PARAMETER :: reltol = 1.0E-8_dpkind
        REAL (dpkind), PARAMETER :: stpmul = 5.0_dpkind
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: which
! ..
! .. Structure Arguments ..
        TYPE (the_distribution), INTENT (IN) :: distrib
        TYPE (zf_locals), INTENT (OUT) :: local
! ..
! .. Executable Statements ..

        CALL set_zero_finder(low_limit=distrib%parameters(which) &
          %low_bound,hi_limit=distrib%parameters(which)%high_bound, &
          abs_step=absstp,rel_step=relstp,step_multiplier=stpmul, &
          abs_tol=atol,rel_tol=reltol,local=local)

      END SUBROUTINE cdf_set_zero_finder

!*********************************************************************

      SUBROUTINE check_complements(x,y,routine_name,x_name,y_name, &
          local_x,local_y,set_values)
! At least one of the optional parameters x and y must be defined in
! the caller.
! If NONE is defined, the program STOPS.
! If ONE parameter is defined:
!   * if:   set_values is FALSE, return to the caller
!     else: set the non-defined parameter to 1 minus the value of the
!          defined parameter.
! If BOTH parameters are defined:
!   * if   : set_values is FALSE, return to the caller
!     else : set local_x=x and local_y=y
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (OUT) :: local_x, local_y
        REAL (dpkind), OPTIONAL, INTENT (IN) :: x, y
        LOGICAL :: set_values
        CHARACTER (*), INTENT (IN) :: routine_name, x_name, y_name
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Local Scalars ..
        INTEGER :: count
! ..
! .. Executable Statements ..

        count = 0

        IF (PRESENT(x)) count = count + 1
        IF (PRESENT(y)) count = count + 1

        SELECT CASE (count)

        CASE (0)

          WRITE (*,90000) routine_name, x_name, y_name
90000     FORMAT (' In routine: ',A/' Neither argument: ',A,' or: ',A, &
            ' is present'/' At least one is required'/ &
            ' Program aborting')
          STOP 'Complement error -- see output'

        CASE (1)

          IF ( .NOT. set_values) RETURN

          IF (PRESENT(x)) THEN
            local_x = x
            local_y = one - x
          ELSE
            local_y = y
            local_x = one - y
          END IF

        CASE (2)

          IF ( .NOT. set_values) RETURN

          local_x = x
          local_y = y

        END SELECT

      END SUBROUTINE check_complements

!*********************************************************************

      FUNCTION dbl_in_range(value,lo,hi,routine_name,arg_name,bad_status, &
          status,bound)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: bound
        REAL (dpkind), INTENT (IN) :: hi, lo, value
        INTEGER, INTENT (IN) :: bad_status
        INTEGER, OPTIONAL, INTENT (OUT) :: status
        CHARACTER (*) :: arg_name, routine_name
! ..
! .. Function Return Value ..
        LOGICAL :: dbl_in_range
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        dbl_in_range = .TRUE.

        IF (value>hi) THEN
          dbl_in_range = .FALSE.
          IF (PRESENT(status)) THEN
            status = bad_status
            IF (PRESENT(bound)) bound = hi
            RETURN
          ELSE
            WRITE (*,90000) routine_name, arg_name, arg_name, value, hi
90000       FORMAT (' Input argument out of range in routine: ', &
              A/' Argument: ',A,' above upper bound'/' The value of ',A, &
              ' is: ',G15.8/' The upper bound on the value is: ', &
              G15.8/' The program is aborting')
            STOP 'Bound error on cdf routine -- see output'
          END IF
        END IF

        IF (value<lo) THEN
          dbl_in_range = .FALSE.
          IF (PRESENT(status)) THEN
            status = bad_status
            IF (PRESENT(bound)) bound = lo
            RETURN
          ELSE
            WRITE (*,90100) routine_name, arg_name, arg_name, value, lo
90100       FORMAT (' Input argument out of range in routine: ', &
              A/' Argument: ',A,' below lower bound'/' The value of ',A, &
              ' is: ',G15.8/' The lower bound on the value is: ', &
              G15.8/' The program is aborting')
            STOP 'Bound error on cdf routine -- see output'
          END IF
        END IF

      END FUNCTION dbl_in_range

!*********************************************************************

      SUBROUTINE error_out(bad_status,bad_bound,status,bound, &
          routine_name)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (IN) :: bad_bound
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: bound
        INTEGER, INTENT (IN) :: bad_status
        INTEGER, OPTIONAL, INTENT (OUT) :: status
        CHARACTER (*), INTENT (IN) :: routine_name
! ..
! .. Local Scalars ..
        CHARACTER (70) :: string
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        IF (PRESENT(status)) THEN
          status = bad_status
          IF (PRESENT(bound)) bound = bad_bound
          RETURN
        ELSE
          string = 'Abort due to error in routine: ' // routine_name
          WRITE (*,*) string
          WRITE (*,*) ' Status: ', bad_status
          WRITE (*,*) ' Bound: ', bad_bound
          STOP &
            'Abort on call to error_out -- see output file for details'
        END IF

      END SUBROUTINE error_out

!*********************************************************************

      FUNCTION evaluate_polynomial(a,x)
!----------------------------------------------------------------------
!              Evaluate a PoLynomial at x
!                              Function
!     Returns:
!          A(1) + A(2)*X + ... + A(N)*X**(N-1)
!                              Arguments
!     A --> Array of coefficients of the polynomial.
!     X --> Point at which the polynomial is to be evaluated.
!----------------------------------------------------------------------
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Function Return Value ..
        REAL (dpkind) :: evaluate_polynomial
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (IN) :: x
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: a(:)
! ..
! .. Local Scalars ..
        REAL (dpkind) :: term
        INTEGER :: i, n
! ..
! .. Intrinsic Functions ..
        INTRINSIC SIZE
! ..
! .. Executable Statements ..

        n = SIZE(a)
        term = a(n)

        DO i = n - 1, 1, -1
          term = a(i) + term*x
        END DO

        evaluate_polynomial = term

      END FUNCTION evaluate_polynomial

!*********************************************************************

      FUNCTION int_in_range(value,lo,hi,routine_name,arg_name,bad_status, &
          status,bound)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: bound
        INTEGER, INTENT (IN) :: bad_status, hi, lo, value
        INTEGER, OPTIONAL, INTENT (OUT) :: status
        CHARACTER (*), INTENT (IN) :: arg_name, routine_name
! ..
! .. Function Return Value ..
        LOGICAL :: int_in_range
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        int_in_range = .TRUE.

        IF (value>hi) THEN
          IF (PRESENT(status)) THEN
            int_in_range = .FALSE.
            status = bad_status
            bound = hi
            RETURN
          ELSE
            WRITE (*,*) ' Input argument error in: ', routine_name
            WRITE (*,*) ' Argument ', arg_name, ' above upper bound'
            WRITE (*,*) arg_name, ' value: ', value
            WRITE (*,*) ' Upper bound: ', hi
            STOP &
              'Stop: uncaught bound error on cdf routine -- see output'
          END IF
        END IF

        IF (value<lo) THEN
          IF (PRESENT(status)) THEN
            int_in_range = .FALSE.
            status = bad_status
            bound = lo
            RETURN
          ELSE
            WRITE (*,*) ' Input argument error in: ', routine_name
            WRITE (*,*) ' Argument ', arg_name, ' below lower bound'
            WRITE (*,*) arg_name, ' value: ', value
            WRITE (*,*) ' Lower bound: ', lo
            STOP &
              'Stop: uncaught bound error on cdf routine -- see output'
          END IF
        END IF

      END FUNCTION int_in_range

!*********************************************************************

      SUBROUTINE set_bound(local,status,bound,routine_name,arg_name)
! Sets bound argument when zero finder wraps off end
! .. Use Statements ..
        USE zero_finder
! ..
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Structure Arguments ..
        TYPE (zf_locals), INTENT (IN) :: local
! ..
! .. Scalar Arguments ..
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: bound
        INTEGER, OPTIONAL, INTENT (OUT) :: status
        CHARACTER (*), INTENT (IN) :: arg_name, routine_name
! ..
! .. Local Scalars ..
        REAL (dpkind) :: left_end, local_bound, right_end
        INTEGER :: local_status
        LOGICAL :: crash_hi, crash_left
        CHARACTER (5) :: up_low
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        CALL final_zf_state(local,local_status,crash_left,crash_hi, &
          left_end,right_end)

        IF (local_status==-1) THEN
          IF (zf_crash_left) THEN
            up_low = 'lower'
            local_status = -50
            local_bound = left_end
          ELSE
            up_low = 'upper'
            local_status = 50
            local_bound = right_end
          END IF

        ELSE
          local_status = 0
        END IF

        IF (local_status==0) THEN
          RETURN
        ELSE
          IF (PRESENT(status)) THEN
            status = local_status
            IF (PRESENT(bound)) bound = local_bound
          ELSE
            IF (local_status==1) THEN
              WRITE (*,90000) routine_name, arg_name, up_low, local_bound

90000         FORMAT (' In routine: ',A/' Calculating: ', &
                A/' Value appears to be beyond ',A, &
                ' bound'/' This bound has the value: ', &
                G15.8/' The program is aborting')
              STOP ' Calculated value out of bounds'
            END IF
          END IF
        END IF

      END SUBROUTINE set_bound

!*********************************************************************

      SUBROUTINE validateparameters(distrib,which,params,status,bound)
!        Generic subroutine which performs validation for
!        ALL the parameters of ALL distributions.
! .. Implicit None Statement ..
! ..  Structures Arguments ..
! .. Array Arguments
! .. Scalar Arguments
        IMPLICIT NONE
! ..
! .. Local Scalars ..
        INTEGER :: i, nparam
        CHARACTER (31) :: tn1, tn2
! ..
! .. Structure Arguments ..
        TYPE (the_distribution) :: distrib
! ..
! .. Scalar Arguments ..
        REAL (dpkind), OPTIONAL, INTENT (OUT) :: bound
        INTEGER, OPTIONAL, INTENT (OUT) :: status
        INTEGER :: which
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: params(6)
! ..
! .. Executable Statements ..

        nparam = distrib%nparam

! Check for which to be within limits

        tn1 = 'which'

        IF ( .NOT. in_range(value=which,lo=1,hi=distrib%max_which, &
          routine_name=distrib%name,arg_name=tn1,bad_status=-1, &
          status=status,bound=bound)) RETURN

! If the unknown is not cdf (which=1), check that cum and ccum add to one

        IF (which/=1) THEN
          tn1 = 'cum'
          tn2 = 'ccum'
          IF ( .NOT. add_to_one(params(1),params( &
            2),distrib%name,tn1,tn2,3,status)) RETURN
        END IF

        DO i = 1, nparam

          IF (which/=distrib%parameters(i)%no_check) THEN
            IF ( .NOT. in_range(value=params(i),lo=distrib%parameters( &
              i)%low_bound,hi=distrib%parameters( &
              i)%high_bound,routine_name=distrib%name, &
              arg_name=distrib%parameters(i)%name,bad_status=-(i+ &
              1),status=status,bound=bound)) RETURN
          END IF

        END DO

      END SUBROUTINE validateparameters

!*********************************************************************

      SUBROUTINE which_miss(which,routine_name,arg_name)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: which
        CHARACTER (*), INTENT (IN) :: arg_name, routine_name
! ..
! .. Executable Statements ..

        WRITE (*,90000) routine_name, which, arg_name, arg_name

90000   FORMAT (' In routine: ',A/' which has value: ',I0, &
          ' indicating that ',A,'is to be calculated'/'But argument: ',A, &
          ' is missing'/'The program is aborting')

        STOP 'Which missing_arg error - see output'

      END SUBROUTINE which_miss

!*********************************************************************

    END MODULE cdf_aux_mod
