    MODULE generate_mod
! .. Default Accessibility ..
      PRIVATE
! ..
! .. Public Statements ..
      PUBLIC :: boot_answers, generate, generate_data, n_rep, &
        set_generate
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
      INTEGER, PARAMETER :: n_rep = 1000
      REAL (dpkind), PARAMETER :: one = 1.0_dpkind
! ..
! .. Local Arrays ..
      REAL (dpkind), POINTER, SAVE :: prob_seen(:,:)
! ..
! .. Intrinsic Functions ..
! Refer to :Local Scalars
      INTRINSIC KIND
! ..
! .. Local Scalars ..
      REAL :: generate_data
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE boot_answers
! .. Use Statements ..
        USE structures_mod
        USE accumulate_mod
! ..
!!! Mu
        CALL stats_from_accum(esum=mu(:,boot,esum), &
          sum_squares=mu(:,boot,ess),n=n_rep,av=mu(:,boot,est), &
          var=mu(:,boot,var),sd=mu(:,boot,sd))

! ..
!!! Raw Frequencies
        CALL stats_from_accum(esum=freq(:,boot,untrans,esum), &
          sum_squares=freq(:,boot,untrans,ess),n=n_rep, &
          av=freq(:,boot,untrans,est),var=freq(:,boot,untrans,var), &
          sd=freq(:,boot,untrans,sd))

!!! Transformed Frequencies
        CALL stats_from_accum(esum=freq(:,boot,trans,esum), &
          sum_squares=freq(:,boot,trans,ess),n=n_rep, &
          av=freq(:,boot,trans,est),var=freq(:,boot,trans,var), &
          sd=freq(:,boot,trans,sd))

!!! Raw Calibrate

        CALL stats_from_accum(esum=calibrate(boot,esum), &
          sum_squares=calibrate(boot,ess),n=n_rep,av=calibrate(boot,est), &
          var=calibrate(boot,var),sd=calibrate(boot,sd))

!!! Raw Total Mutants

        CALL stats_from_accum(esum=mutant(boot,untrans,esum), &
          sum_squares=mutant(boot,untrans,ess),n=n_rep, &
          av=mutant(boot,untrans,est),var=mutant(boot,untrans,var), &
          sd=mutant(boot,untrans,sd))

!!! Transformed Total Mutants

        CALL stats_from_accum(esum=mutant(boot,trans,esum), &
          sum_squares=mutant(boot,trans,ess),n=n_rep, &
          av=mutant(boot,trans,est),var=mutant(boot,trans,var), &
          sd=mutant(boot,trans,sd))

        RETURN

      END SUBROUTINE boot_answers

!*********************************************************************

      SUBROUTINE generate
! .. Use Statements ..
        USE random_binomial_mod
        USE structures_mod
! ..
! .. Local Scalars ..
        REAL :: prob
        INTEGER :: generated, i_allele, i_dna, n
! ..
! .. Intrinsic Functions ..
        INTRINSIC INT, REAL
! ..
!!! Generates one random experiment using design and parameter values
!!! from set_generate.  Puts data in seen(:,:,boot) and unseen(:,:,boot).
        DO i_dna = 1, n_dna

          DO i_allele = 1, n_allele

            n = INT(n_well(i_dna))

            prob = REAL(prob_seen(i_dna,i_allele))

            generated = random_binomial(n,prob)

            seen(i_dna,i_allele,boot) = REAL(generated,kind=dpkind)

            unseen(i_dna,i_allele,boot) = REAL(n_well(i_dna),kind=dpkind) &
              - seen(i_dna,i_allele,boot)

          END DO

        END DO

        RETURN

      END SUBROUTINE generate

!*********************************************************************

      SUBROUTINE set_generate(from_truth)
! .. Use Statements ..
        USE resize_mod
        USE structures_mod
! ..
! .. Scalar Arguments ..
        LOGICAL, INTENT (IN) :: from_truth
! ..
! .. Intrinsic Functions ..
        INTRINSIC EXP
! ..
! .. Local Scalars ..
        INTEGER :: i_allele, i_dna
! ..
!----------------------------------------------------------------------
!    Generates a single data set according to the input frequency of each
!    allele, the experimental design, and calibrate.
!                               ARGUMENTS
!    n_dna --> The number of runs with different dna amounts
!    n_allele --> The total number of alleles seen
!    n_well --> The number of repetitions of the i'th run
!    dna --> The i'th element is the amount of dna in the i'th run
!    freq --> The i'th element is the frequency of the i'th allele.
!             The freq list should add to one but this is not checked.
!    calibrate --> calibrate user dna units are one allele equivalent
!    seen <-- Matrix of number of times out of n_well(i) that allele(j)
!    was seen
!______________________________________________________________________
!!! Allocate prob_seen
        CALL size_matrix(prob_seen,n_dna,n_allele)

!!! Calculate prob_seen

        IF (from_truth) THEN

          DO i_dna = 1, n_dna

            DO i_allele = 1, n_allele

              prob_seen(i_dna,i_allele) = dna(i_dna)*true_freq(i_allele)

            END DO

          END DO

          prob_seen = true_calibrate*prob_seen

          prob_seen = one - EXP(-prob_seen)

        ELSE

          prob_seen = seen(:,:,data)/(seen(:,:,data)+unseen(:,:,data))

        END IF

        RETURN

      END SUBROUTINE set_generate

!*********************************************************************

    END MODULE generate_mod
