    MODULE one_data_set_mod
! .. Default Accessibility ..
      PUBLIC
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
      REAL (dpkind), PARAMETER :: half = 0.5_dpkind
      REAL (dpkind), PARAMETER :: zero = 0.0_dpkind
! ..
! .. Intrinsic Functions ..
      INTRINSIC KIND
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE bootstrap(from_truth,write_simulations,sim_unit)
!!! Do entire parametric bootstrap
!!! Assumes the data has been fit
! .. Use Statements ..
        USE structures_mod
        USE set_seeds_mod
        USE generate_mod
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: sim_unit
        LOGICAL, INTENT (IN) :: from_truth, write_simulations
! ..
! .. Local Scalars ..
        INTEGER :: i_rep
! ..
        CALL set_seeds(which=1)

        CALL set_generate(from_truth=from_truth)

        DO i_rep = 1, n_rep

          CALL generate

          CALL fit_one(bootstrap=.TRUE.,write_simulations= &
            write_simulations,sim_unit=sim_unit)

        END DO

        CALL boot_answers

      END SUBROUTINE bootstrap

!*********************************************************************

      SUBROUTINE fit_one(bootstrap,write_simulations,sim_unit)
! .. Use Statements ..
        USE accumulate_mod
        USE fit_mu_mod
        USE fit_freq_mod
        USE structures_mod
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: sim_unit
        LOGICAL, INTENT (IN) :: bootstrap, write_simulations
! ..
! .. Local Scalars ..
        INTEGER :: ix_data_boot
! ..
! .. Intrinsic Functions ..
        INTRINSIC SQRT
! ..
        ix_data_boot = data

        IF (bootstrap) ix_data_boot = boot

!!! Modify data to eliminate infinities

        CALL modify_data(seen_in=seen(:,:,ix_data_boot), &
          unseen_in=unseen(:,:,ix_data_boot),seen_out=seen(:,:,shrink), &
          unseen_out=unseen(:,:,shrink))

!!! Fit mus

        CALL fit_all_mu(seen=seen(:,:,shrink),unseen=unseen(:,:,shrink), &
          mu_init_est=mu(:,ix_data_boot,init_est), &
          mu_final_est=mu(:,ix_data_boot,est),mu_est_var=mu(:, &
          ix_data_boot,var))

        mu(:,ix_data_boot,sd) = SQRT(mu(:,ix_data_boot,var))

!!! Fit frequencies

        CALL est_var_freq(est_mu=mu(:,ix_data_boot,est), &
          var_mu=mu(:,ix_data_boot,var),est_freq=freq(:,ix_data_boot, &
          untrans,est),var_freq=freq(:,ix_data_boot,untrans,var), &
          sd_freq=freq(:,ix_data_boot,untrans,sd))

        freq(:,ix_data_boot,trans,est) = arcsin(p=freq(:,ix_data_boot, &
          untrans,est))

        freq(:,ix_data_boot,trans,var) = asy_var_arcsin(p=freq(:, &
          ix_data_boot,untrans,est),var_p=freq(:,ix_data_boot,untrans,var &
          ))

        freq(:,ix_data_boot,trans,sd) = SQRT(freq(:,ix_data_boot,trans, &
          var))

        CALL est_var_calibrate(est_mu=mu(:,ix_data_boot,est), &
          var_mu=mu(:,ix_data_boot,var),est_calibrate=calibrate( &
          ix_data_boot,est),var_calibrate=calibrate(ix_data_boot,var), &
          sd_calibrate=calibrate(ix_data_boot,sd))

!!! Total Mutants

        CALL est_var_mut(est_mu=mu(:,ix_data_boot,est), &
          var_mu=mu(:,ix_data_boot,var),est_mut_freq=mutant(ix_data_boot, &
          untrans,est),var_mut_freq=mutant(ix_data_boot,untrans,var), &
          sd_mut_freq=mutant(ix_data_boot,untrans,sd))

        mutant(ix_data_boot,trans,est) = arcsin(p=mutant(ix_data_boot, &
          untrans,est))

        mutant(ix_data_boot,trans,var) = asy_var_arcsin(p=mutant( &
          ix_data_boot,untrans,est),var_p=mutant(ix_data_boot,untrans,var &
          ))

        mutant(ix_data_boot,trans,sd) = SQRT(mutant(ix_data_boot,trans, &
          var))

        IF (bootstrap) THEN

          CALL accumulate(term=mu(:,boot,est),esum=mu(:,boot,esum), &
            sum_squares=mu(:,boot,ess))

          CALL accumulate(term=freq(:,boot,untrans,est), &
            esum=freq(:,boot,untrans,esum),sum_squares=freq(:,boot, &
            untrans,ess))

          CALL accumulate(term=freq(:,boot,trans,est), &
            esum=freq(:,boot,trans,esum),sum_squares=freq(:,boot,trans, &
            ess))

          CALL accumulate(term=calibrate(boot,est), &
            esum=calibrate(boot,esum),sum_squares=calibrate(boot,ess))

          CALL accumulate(term=mutant(boot,untrans,est), &
            esum=mutant(boot,untrans,esum),sum_squares=mutant(boot, &
            untrans,ess))

          CALL accumulate(term=mutant(boot,trans,est), &
            esum=mutant(boot,trans,esum),sum_squares=mutant(boot,trans, &
            ess))

          IF (write_simulations) THEN

            WRITE (sim_unit,90000) calibrate(boot,est)

90000       FORMAT ('Cal  ',F10.4)

            WRITE (sim_unit,90100) mu(:,boot,est)

90100       FORMAT ('Mu   ',8F8.4)

            WRITE (sim_unit,90200) freq(:,boot,untrans,est)

90200       FORMAT ('Freq ',10(1X,F6.4))

          END IF

        END IF

        RETURN

      END SUBROUTINE fit_one

!*********************************************************************

      SUBROUTINE modify_data(seen_in,unseen_in,seen_out,unseen_out)
!!!----------------------------------------------------------------------
!!!
!!! Modifies seen_in and unseen_in so that unseen is not uniformly 0 for
!!! any allele.  Puts modified data in seen_out and unseen_out
!!! At the moment (5 Dec 02) the only modification is to decrease the
!!! seen value by 1/2 at the lowest dna amount and increase unseed
!!! at the same point correspondingly.
!!!
!!!----------------------------------------------------------------------
! .. Use Statements ..
        USE structures_mod, ONLY : n_dna, n_allele, dna
! ..
! .. Parameters ..
        REAL (dpkind), PARAMETER :: half = 0.5_dpkind
! ..
! .. Array Arguments ..
        REAL (dpkind), INTENT (IN) :: seen_in(1:n_dna,1:n_allele), &
          unseen_in(1:n_dna,1:n_allele)
        REAL (dpkind), INTENT (OUT) :: seen_out(1:n_dna,1:n_allele), &
          unseen_out(1:n_dna,1:n_allele)
! ..
! .. Local Scalars ..
        INTEGER :: ixmin, i_allele
        LOGICAL :: ok
! ..
! .. Local Arrays ..
        INTEGER :: min_dna_loc(1)
! ..
! .. Intrinsic Functions ..
        INTRINSIC ALL, MINLOC
! ..
        seen_out = seen_in
        unseen_out = unseen_in

!!!  See whether unseen is zero anywhere
!!!
!!!  If not, no adjustment is necessary

        ok = ALL(unseen_in>zero)

        IF (ok) RETURN

!!! Unseen is zero  somewhere -- perhaps modifications must be made

!!! Check each allele in turn

ALLELE_LOOP: DO i_allele = 1, n_allele

          ok = ALL(unseen_out(:,i_allele)>zero)

          IF (ok) CYCLE ALLELE_LOOP

!!! Unseen uniformly zero in this allele -- adjustment must be made

          min_dna_loc = MINLOC(dna)

          ixmin = min_dna_loc(1)

          seen_out(ixmin,i_allele) = seen_out(ixmin,i_allele) - half

          unseen_out(ixmin,i_allele) = unseen_out(ixmin,i_allele) + half

        END DO ALLELE_LOOP

        RETURN

      END SUBROUTINE modify_data

!*********************************************************************

    END MODULE one_data_set_mod
