    MODULE problem_in_mod
! .. Default Accessibility ..
      PRIVATE
! ..
! .. Public Statements ..
      PUBLIC :: end_line, generate_data, generate_parameters_in, &
        interactive_read, read_mjs_one, read_one_bat
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
      INTEGER, PARAMETER :: max_allele_in = 50, max_dna_in = 50
! ..
! .. Intrinsic Functions ..
! Refer to :Local Scalars
      INTRINSIC KIND
! ..
! .. Local Scalars ..
      REAL :: generate_data
      LOGICAL :: progenitor_error, qstart
! ..
! .. Local Arrays ..
      INTEGER :: progenitor_ix_in(2)
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE check_progenitor(p_size,allele_size,error)
!!! Converts size of progenitors to indices in allele_size
! .. Scalar Arguments ..
        LOGICAL, INTENT (OUT) :: error
! ..
! .. Array Arguments ..
        INTEGER, INTENT (IN) :: allele_size(:), p_size(2)
! ..
! .. Local Scalars ..
        INTEGER :: i_allele_group, n_allele
        LOGICAL :: progenitor1_ok, progenitor2_ok
! ..
! .. Intrinsic Functions ..
        INTRINSIC SIZE
! ..
        progenitor1_ok = .FALSE.

        progenitor2_ok = .FALSE.

        n_allele = SIZE(allele_size)

        DO i_allele_group = 1, n_allele

          IF (p_size(1)==allele_size(i_allele_group)) THEN
            progenitor1_ok = .TRUE.
          END IF

          IF (p_size(2)==allele_size(i_allele_group)) THEN
            progenitor2_ok = .TRUE.
          END IF

        END DO

        error = .NOT. (progenitor1_ok .AND. progenitor2_ok)

        RETURN

      END SUBROUTINE check_progenitor

!*********************************************************************

      SUBROUTINE end_line(str)
!!! Looks for / or #
!!! If found blanks this character and all others to the end of
!!! the line.
! .. Parameters ..
        CHARACTER (2), PARAMETER :: terminators = '/#'
! ..
! .. Scalar Arguments ..
        CHARACTER (*), INTENT (INOUT) :: str
! ..
! .. Local Scalars ..
        INTEGER :: end_pos
! ..
! .. Intrinsic Functions ..
        INTRINSIC SCAN
! ..
        end_pos = SCAN(str,terminators)
        IF (end_pos<1) RETURN
        str(end_pos:) = ' '
      END SUBROUTINE end_line

!*********************************************************************

      SUBROUTINE error_out(errno,i_val,val,line)
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: errno, line
        INTEGER, OPTIONAL, INTENT (IN) :: i_val
        CHARACTER (*), OPTIONAL :: val
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT, TRIM
! ..
        WRITE (*,90000) line

90000   FORMAT (' Error on line number: ',I7,' of input.')

        IF (PRESENT(i_val)) WRITE (*,90100) i_val, TRIM(val)

90100   FORMAT (' The ordinal number of the value being read &
          &from the line is: ',I5/' The value being processed was:'/70( &
          '$')/1X,A/70('$')//' The error is indicated below.'/ &
          ' Please fix the data and rerun the program.'/)

        SELECT CASE (errno)

        CASE (1)

          WRITE (*,90200)

90200     FORMAT (' Not enough data items on the line -- at &
            &least 6 items required')

        CASE (2)

          WRITE (*,90300)

90300     FORMAT (' An integer value was expected here and &
            &something else found.')

        CASE (3)

          WRITE (*,90400)

90400     FORMAT (' A numeric value was expected here and something &
            &else found.')

        CASE (4)

          WRITE (*,90500)

90500     FORMAT (' More than 500 values on this line')

        CASE (5)

          WRITE (*,90600)

90600     FORMAT (' Odd number of entries in what should be pairs:'/ &
            ' (size, number seen)')

        CASE (6)

          WRITE (*,90700)

90700     FORMAT (' Some number of alleles seen either negative or else'/ &
            ' greater than the number of wells.')

        CASE (7)

          WRITE (*,90800)

90800     FORMAT (' Progenitor number on this line doesn''t match that'/ &
            ' on the first line of the data. ')

        CASE (8)

          WRITE (*,90900)

90900     FORMAT (' Progenitor size not found in input ''values &
            &of allele sizes.')

        CASE (9)

          WRITE (*,91000)

91000     FORMAT (' Number of alleles not the same as on the first '/ &
            ' data record')

        CASE (10)

          WRITE (*,91100)

91100     FORMAT (' Size of allele differs from that specified &
            &on the first data record')

        CASE (11)

          WRITE (*,91200) max_dna_in

91200     FORMAT (' The maximum number of runs (dna amounts) is: ', &
            I3/' Attempt made to exceed this maximum.')

        END SELECT

        STOP 'Program Aborted -- Error in Data'

      END SUBROUTINE error_out

!*********************************************************************

      SUBROUTINE generate_parameters_in(from_truth,write_simulations, &
          rep_unit)
! .. Use Statements ..
        USE generate_mod
        USE get_numbers_mod
        USE one_data_set_mod
        USE print_it
        USE random_binomial_mod
        USE results_out_mod
        USE set_seeds_mod
        USE structures_mod
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: rep_unit
        LOGICAL, INTENT (OUT) :: from_truth, write_simulations
! ..
! .. Local Scalars ..
        REAL (dpkind) :: tot_freq
        INTEGER :: i, num_well
        LOGICAL :: error
! ..
! .. Intrinsic Functions ..
        INTRINSIC SUM
! ..
        num_subs = 0

        message_format = '(5X,''Enter (1) The number of dna &
          &amounts to be simulated''/5X,''      (2) The number &
          &of alleles to be simulated'')'

        CALL print_message_format

        READ (*,*) n_dna, n_allele

        CALL initialize

        DO i = 1, n_allele
          allele_size(i) = i
        END DO

        message_format = '(/5X,''This program does not allow &
          & the number of wells (replicates) per''/5X,''run &
          &to vary from run to run.''//5X,''Enter the number &
          &of wells per run for the simulated data.''/)'

        CALL get_numbers(message_format,lo=1,hi=1000,x=num_well)

        n_well = num_well

90000   FORMAT (' Enter the frequencies of the ',I3, &
          ' alleles.'/ &
          '  The frequencies will be normalized to add to one.'/)

        WRITE (*,90000) n_allele

        READ (*,*) true_freq

        tot_freq = SUM(true_freq)

        true_freq = true_freq/tot_freq

        num_subs = 2

        sub_pos(1:4) = (/ 60, 62, 107, 109 /)

        WRITE (sub_strings(1),'(I3)') n_dna

        sub_strings(2) = sub_strings(1)

        message_format = '(/5X,''The is one DNA amount per &
          &run and you have spocified %%% runs for this''/5X,''experiment.&
          &''//5X,''Enter %%% DNA amounts, one for each run.'')'

        CALL edit_message_format

        CALL get_numbers(format=message_format,lo=(/0.001_dpkind/), &
          hi=(/10000.0_dpkind/),x=dna)

        message_format = '(/5X,''The  calibration  parameter &
          & is  the  number  of  g.e.''''s  in  the''/5X,''experimenter'''&
          &'s DNA unit.''//5X,''Enter the calibration parameter &
          &for the simulated data.''/)'

        CALL get_numbers(format=message_format,lo=0.001_dpkind, &
          hi=1000.0_dpkind,x=true_calibrate)

        message_format = '(/5X,''Enter the indices (ordinal &
          &number) of the two progenitor alleles.''/5X,''If &
          & the simulated  sample  is homozygous,  enter  the &
          &same  index''/5X,''twice.''/)'

        CALL get_numbers(format=message_format,lo=(/1/),hi=(/n_allele/), &
          x=progenitor)

        message_format = '(/5X,''Do you  want bootstrapping &
          &to  use the values you  entered rather''/5X,''than &
          &those from the first data set generated?''//5X,''Enter &
          &''''y'''' for this option else ''''n''''.''/)'

        CALL get_yn(message_format,from_truth,error)

        message_format = '(/5X,''Do  you want  the  simulated &
          &values  (mu, calibration,  frequency''/5X,''estimation) &
          &written to a file for further processing?''//5X,''Enter &
          &''''y'''' for this option else ''''n''''.''//5X,''This &
          & option is  usually  used only  by  those investigating &
          & the''/5X,''properties of  the statistical method &
          & -- not by  those analysing''/5X,''data.''/)'

        CALL get_yn(message_format,write_simulations,error)

        CALL print_generate_params(rep_unit)

        CALL set_seeds(which=1)

        CALL set_generate(from_truth)

        CALL generate

        seen(:,:,data) = seen(:,:,boot)

        unseen(:,:,data) = unseen(:,:,boot)

      END SUBROUTINE generate_parameters_in

!*********************************************************************

      SUBROUTINE interactive_read
! .. Use Statements ..
        USE data_in_struct_mod
        USE get_numbers_mod
        USE print_it
! ..
! .. Local Scalars ..
        INTEGER :: i_run, n_write
! ..
! .. Intrinsic Functions ..
        INTRINSIC MIN
! ..
        num_subs = 0

        message_format = '(/5X,''GENERAL INFORMATION ON INTERACTIVE &
          &DATA ENTRY:''//5X,''RUNS: A run  consists of a number &
          & of  wells examined at a  particular''/5X,''genome &
          &equivalent  amount of  dna.   Different runs can &
          &have  the same''/5X,''amount of dna, but this is &
          &unusual.''//5X,''ALLELES: Alleles are  identified &
          &through integer  numbers that we term''/5X,''size. &
          & The size is only the label for the allele -- it &
          &is not used in''/5X,''any calculation.''//5X,''You &
          &can enter information on  alleles that are  not &
          &seen at all in the''/5X,''experiment if this makes &
          &transcription easier.  However, these alleles''/5X,''are &
          &not included in the analysis.''//5X,''GENERAL: Lists &
          &of numbers can be entered on several lines (the &
          &program''/5X,''keeps reading until it has enough &
          &numbers then quits reading.  Numbers''/5X,''themselves &
          &cannot span lines.  Thus 123 CANNOT be written as &
          &1 at the''/5X,''end of a line and 23 at the beginning &
          &of the next line.''/)'

        CALL print_message_format(force=.TRUE.)

!!! Number of runs

        message_format = '(/5X,''Enter the number of runs (number &
          &of dna amounts):''/5X,''Range (1 to 50)''/)'

        CALL get_numbers(format=message_format,lo=1,hi=max_dna_in, &
          x=n_dna_in)

!!! Number of alleles seen

        message_format = '(/5X,''Enter the number of alleles &
          &seen:''/5X,''Range (1 to 50)''/)'

        CALL get_numbers(format=message_format,lo=1,hi=max_allele_in, &
          x=n_allele_in)

!!! Sizes of alleles

        num_subs = 1

        sub_pos(1:2) = (/ 39, 40 /)

! Assign sub_strings(1)

!sub_string_len = 2
        WRITE (sub_strings(1),'(I2)') n_allele_in


        message_format = '(/5X,''Enter the  integer sizes of &
          &the %%  alleles.  Each size can  be in the''/5X,''range &
          &of 1-999.''/)'

        CALL edit_format(message_format)

        CALL get_numbers(format=message_format,lo=(/1/),hi=(/999/), &
          x=size_in(1:n_allele_in))

!!! Progenitor allele sizes

        num_subs = 0

        message_format = '(/5X,''Enter the size of the two &
          &progenitor alleles.  If the subject is homozygous''/5X,''enter &
          &the size of the one progenitor allele twice.''//5X,''Enter &
          &two numbers.''/)'

        CALL get_numbers(format=message_format,lo=(/1/),hi=(/999/), &
          x=progenitors_in)

!!! Check that progenitors are on the list of alleles

        CALL check_progenitor(progenitors_in,size_in,progenitor_error)

        IF (progenitor_error) THEN
          WRITE (*,90000)
        END IF

90000   FORMAT (' Progenitor allele size does not appear on &
          &the list of allele sizes'/' The program is aborting.')

!!! Dna amount for each run

        num_subs = 1

        sub_pos(1:2) = (/ 106, 107 /)

! Assign sub_strings(1)

! sub_string_len = 2
        WRITE (sub_strings(1),'(I2)') n_dna_in

        message_format = '(/5X,''For  each  run (dna  amount), &
          & enter the  dna  amount  in your  genome''/5X,''equivalent &
          &units.''/5X,''%% numbers should be entered -- several &
          &lines can be used if desired''/)'

        CALL edit_format(message_format)

        CALL get_numbers(format=message_format,lo=(/0.001_dpkind/), &
          hi=(/1000.0_dpkind/),x=dna_in(1:n_dna_in))

        num_subs = 1

        sub_pos(1:2) = (/ 78, 79 /)

! Assign sub_strings(1)

! sub_string_len = 2
        WRITE (sub_strings(1),'(I2)') n_dna_in

        message_format = '(/5X,''For each run (dna amount), &
          &enter the number of wells (replicates)''/5X,''%% &
          &numbers should be entered -- several lines can be &
          &used if desired''/)'

        CALL edit_format(message_format)

        CALL get_numbers(format=message_format,lo=(/1/),hi=(/500/), &
          x=n_well_in(1:n_dna_in))

        num_subs = 5

        sub_pos(1:10) = (/ 27, 28, 51, 56, 93, 96, 289, 292, 328, 329 /)

        DO i_run = 1, n_dna_in

          WRITE (*,90100) i_run, dna_in(i_run), n_well_in(i_run), &
            n_allele_in, n_well_in(i_run)

90100     FORMAT (// &
            'You are entering the number of times each allele seen:'/ &
            20('-')/'Run number ',I3,' dna amount (your ge) ',F6.2, &
            ' number of wells ',I3/'Enter ',I2, &
            ' numbers, each between 0 and ',I3,'.'/20('-')/ &
            'The sizes of the first 20 alleles are printed below:')

          n_write = MIN(20,n_allele_in)

          WRITE (*,90200) size_in(1:n_write)

90200     FORMAT (20(1X,I3))

          CALL get_numbers(lo=(/0/),hi=(/n_well_in(i_run)/), &
            x=seen_in(i_run,1:n_allele_in))

        END DO

        CALL values_to_structures

      END SUBROUTINE interactive_read

!*********************************************************************

      SUBROUTINE read_mjs_one(unit)
! .. Use Statements ..
        USE data_in_struct_mod
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: unit
! ..
! .. Local Scalars ..
        REAL (dpkind) :: dval, real_in
        REAL :: rval
        INTEGER :: indov, ival, lcval, line, n_allele_vals, &
          n_integers_in, n_val_in, up
        LOGICAL :: is_more
        CHARACTER (2) :: curtyp
        CHARACTER (500) :: cval, line_in
! ..
! .. Local Arrays ..
        INTEGER :: integers_in(500)
! ..
        up = max_dna_in + 1

        DO line = 1, up

          CALL read_decipher_line(unit,line,is_more)

          IF ( .NOT. is_more) THEN

            n_dna_in = line - 1

            EXIT

          END IF

          IF (line>max_dna_in) CALL error_out(errno=11,line=line)

          CALL check_and_process_values(line)

        END DO

        CALL values_to_structures

        RETURN

      CONTAINS

!.....................................................................

        SUBROUTINE blank_chars(str)
!!! Replaces the characters quote, comma, and anything above ascii
!!! 255 with a blank.
! .. Parameters ..
          CHARACTER (1), PARAMETER :: blank = ' ', comma = ',', &
            quote = '"'
! ..
! .. Scalar Arguments ..
          CHARACTER (*) :: str
! ..
! .. Local Scalars ..
          INTEGER :: i, lstr
! ..
! .. Intrinsic Functions ..
          INTRINSIC IACHAR, LEN_TRIM
! ..
          lstr = LEN_TRIM(str)

          DO i = 1, lstr

            IF ((str(i:i)==quote) .OR. (str(i:i)==comma) .OR. (IACHAR( &
              str(i:i))>127)) str(i:i) = blank

          END DO

          RETURN

        END SUBROUTINE blank_chars

!.....................................................................

        SUBROUTINE check_and_process_values(line)
! .. Use Statements ..
          USE data_in_struct_mod
! ..
! .. Scalar Arguments ..
          INTEGER, INTENT (IN) :: line
! ..
! .. Local Scalars ..
          INTEGER :: ix, i_allele_group
          LOGICAL :: progenitor1_ok, progenitor2_ok, progenitor_error
! ..
          progenitor1_ok = .FALSE.

          progenitor2_ok = .FALSE.

FIRST_LINE_IF: IF (line==1) THEN

            progenitors_in(1) = integers_in(1)
            progenitors_in(2) = integers_in(2)

            n_well_in(line) = integers_in(3)

            dna_in(line) = real_in

            n_allele_vals = n_integers_in - 3

            n_allele_in = n_allele_vals/2

            IF ((n_allele_vals-n_allele_in*2)/=0) CALL error_out(errno=5, &
              line=line)

            ix = 3

            DO i_allele_group = 1, n_allele_in

              ix = ix + 1

              size_in(i_allele_group) = integers_in(ix)

              ix = ix + 1

              seen_in(line,i_allele_group) = integers_in(ix)

              IF ((seen_in(line,i_allele_group)<0) .OR. (seen_in(line, &
                  i_allele_group)>n_well_in(line))) THEN
                CALL error_out(errno=6,line=line)
              END IF

            END DO

            CALL check_progenitor(progenitors_in,size_in, &
              progenitor_error)

            IF (progenitor_error) CALL error_out(errno=8,line=line)

          ELSE FIRST_LINE_IF !!! Not first line of data file

            IF ((progenitors_in(1)/=integers_in(1)) .OR. (progenitors_in( &
              2)/=integers_in(2))) CALL error_out(errno=7,line=line)

            dna_in(line) = real_in

            n_well_in(line) = integers_in(3)

            IF ((n_integers_in-3)/=2*n_allele_in) CALL error_out(errno=9, &
              line=line)

            ix = 3

            DO i_allele_group = 1, n_allele_in

              ix = ix + 1

              IF (size_in(i_allele_group)/=integers_in(ix)) &
                CALL error_out(errno=10,line=line)

              ix = ix + 1

              seen_in(line,i_allele_group) = integers_in(ix)

              IF ((seen_in(line,i_allele_group)<0) .OR. (seen_in(line, &
                i_allele_group)>n_well_in(line))) CALL error_out(errno=6, &
                line=line)

            END DO

          END IF FIRST_LINE_IF

        END SUBROUTINE check_and_process_values

!.....................................................................

        SUBROUTINE read_decipher_line(unit,line_no,is_more)
!!! Reads and obtains values from a single line of input
! .. Use Statements ..
          USE qlex_mod
! ..
! .. Scalar Arguments ..
          INTEGER, INTENT (IN) :: line_no, unit
          LOGICAL :: is_more
! ..
! .. Local Scalars ..
          INTEGER :: ios
          LOGICAL :: is_more_on_line
! ..
! .. Parameters ..
          INTEGER, PARAMETER :: max_val_in = 54
! ..
! .. Intrinsic Functions ..
          INTRINSIC TRIM
! ..
          line_in = ' '

          READ (unit,'(A)',iostat=ios) line_in

          IF (ios==0) THEN
            is_more = .TRUE.
          ELSE
            is_more = .FALSE.
            RETURN
          END IF

!!! Blank trailing chars ending in / or #

          CALL end_line(line_in)

!!! Get rid of ", , , and strange characters.  What is left should be
!!! a string of numbers

          CALL blank_chars(line_in)
          WRITE (*,'(''$'',A,''$'')') TRIM(line_in)

!!! Okay -- read everything

          n_val_in = 0
          n_integers_in = 0

          qstart = .TRUE.

SPLIT_LINE: DO

            is_more_on_line = qlex(line_in,qstart,curtyp,cval,lcval,ival, &
              rval,dval,indov)

MORE_VALS:  IF (is_more_on_line) THEN
              n_val_in = n_val_in + 1

              IF (n_val_in>max_val_in) CALL error_out(errno=11, &
                line=line_no)

              IF (n_val_in==4) THEN ! value 4 is dna which is real
                IF (curtyp=='IN' .OR. curtyp=='RL') THEN
                  real_in = dval
                ELSE
                  CALL error_out(errno=3,i_val=n_val_in,val=cval, &
                    line=line_no)
                END IF

              ELSE ! everything else should be an integer

                IF (curtyp/='IN') THEN
                  CALL error_out(errno=2,i_val=n_val_in,val=cval, &
                    line=line_no)
                ELSE
                  n_integers_in = n_integers_in + 1
                  integers_in(n_integers_in) = ival
                END IF

              END IF

            ELSE MORE_VALS ! No more on line

              EXIT SPLIT_LINE

            END IF MORE_VALS

          END DO SPLIT_LINE

          RETURN

        END SUBROUTINE read_decipher_line

!.....................................................................

      END SUBROUTINE read_mjs_one

!*********************************************************************

      SUBROUTINE read_one_bat(data_unit)
!  Reads one data set in batch format from data_unit
! .. Use Statements ..
        USE data_in_struct_mod
        USE get_numbers_mod
        USE print_it
        USE qlex_mod
! ..
! .. Parameters ..
        INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
        INTEGER, PARAMETER :: max_n_obtained = 100, max_string = 256
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: data_unit
! ..
! .. Local Scalars ..
        REAL (dpkind) :: dval
        REAL :: rval
        INTEGER :: indov, ios, ival, lcval, n_run_in, n_type_processed
        LOGICAL :: is_more, more_lines
        CHARACTER (2) :: curtyp
        CHARACTER (max_string) :: cval
        CHARACTER (max_string) :: line, line_read
! ..
! .. Local Arrays ..
        INTEGER :: numbers_obtained(max_n_obtained)
! ..
        n_type_processed = 0
        n_run_in = 0

LINE_LOOP: DO ! Read data unil there is no more

          CALL get_line(data_unit,line,more_lines)

          IF ( .NOT. more_lines) EXIT

          qstart = .TRUE.

          is_more = qlex(line,qstart,curtyp,cval,lcval,ival,rval,dval, &
            indov)

          IF (cval=='nallele') THEN
            n_type_processed = n_type_processed + 1
            IF (n_type_processed/=1) CALL bat_error_out(-1)
            CALL process_nallele
          ELSE IF (cval=='nrun') THEN
            n_type_processed = n_type_processed + 1
            IF (n_type_processed/=2) CALL bat_error_out(-2)
            CALL process_nrun
          ELSE IF (cval=='nwell') THEN
            n_type_processed = n_type_processed + 1
            IF (n_type_processed/=3) CALL bat_error_out(-3)
            CALL process_nwell
          ELSE IF (cval=='allelesizes') THEN
            n_type_processed = n_type_processed + 1
            IF (n_type_processed/=4) CALL bat_error_out(-4)
            CALL process_allelesizes
          ELSE IF (cval=='progenitor') THEN
            n_type_processed = n_type_processed + 1
            IF (n_type_processed/=5) CALL bat_error_out(-5)
            CALL process_progenitor
          ELSE IF (cval=='run') THEN
            n_type_processed = n_type_processed + 1
            IF (n_type_processed<=5) CALL bat_error_out(-6)
            n_run_in = n_run_in + 1
            IF (n_run_in>n_dna_in) CALL bat_error_out(10)
            CALL process_run
          ELSE
            CALL bat_error_out(0)
          END IF

        END DO LINE_LOOP

        IF (n_run_in/=n_dna_in) CALL bat_error_out(11)

        CALL values_to_structures

      CONTAINS

!.....................................................................

        SUBROUTINE alphanum(string)
!!! Blanks out all characters that are not alpha or numberic or '.'
! .. Parameters ..
          CHARACTER (*), PARAMETER :: allowable_chars = 'abcdefghijklmnopq&
            &rstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.'
          CHARACTER (1), PARAMETER :: blank = ' '
! ..
! .. Scalar Arguments ..
          CHARACTER (*), INTENT (INOUT) :: string
! ..
! .. Local Scalars ..
          INTEGER :: ix, len
! ..
! .. Intrinsic Functions ..
          INTRINSIC INDEX, LEN_TRIM
! ..
          len = LEN_TRIM(string)

          DO ix = 1, len

            IF (INDEX(allowable_chars,string(ix:ix))>0) THEN
              CYCLE
            ELSE
              string(ix:ix) = blank
            END IF

            RETURN

          END DO

        END SUBROUTINE alphanum

!.....................................................................

        SUBROUTINE bat_error_out(errno)
! .. Scalar Arguments ..
          INTEGER, INTENT (IN) :: errno
! ..
! .. Local Scalars ..
          CHARACTER (max_string) :: error_message
! ..
! .. Intrinsic Functions ..
          INTRINSIC TRIM
! ..
          WRITE (*,90000) TRIM(line_read), TRIM(line)

90000     FORMAT ('Error when processing the following line: '/A// &
            ' After processing, the line was as follows:'/'$',A,'$'/)

          SELECT CASE (errno)

          CASE (-1)
            error_message = &
              'Expecting an "NALLELE" record when this line read.'
          CASE (-2)
            error_message = &
              'Expecting an "NRUN" record when this line read.'
          CASE (-3)
            error_message = &
              'Expecting an "NWELL" record when this line read.'
          CASE (-4)
            error_message = &
              'Expecting a "RUN" record when this line read.'
          CASE (8)
            error_message = &
              'A progenitor size ddes not match any allele size.'
          CASE (10)
            error_message = 'Too many RUN records'
          CASE (0)
            error_message = 'Unrecognized identifier on record'
          CASE (101)
            error_message = &
              'Non-integer data item on record where integer expected'
          CASE (102)
            error_message = &
              'Nonblank fields on record following last integer needed'
          CASE DEFAULT
            WRITE (*,90100) errno
90100       FORMAT (' No message provided for this error message &
              &in routine bat_error_out'/' The error number is: ',I10)
          END SELECT

          WRITE (*,'(A)') TRIM(error_message)

          RETURN

        END SUBROUTINE bat_error_out

!.....................................................................

        SUBROUTINE get_line(data_unit,line,more_lines)
! .. Scalar Arguments ..
          INTEGER, INTENT (IN) :: data_unit
          LOGICAL, INTENT (OUT) :: more_lines
          CHARACTER (max_string), INTENT (OUT) :: line
! ..
! .. Intrinsic Functions ..
          INTRINSIC LEN_TRIM
! ..
READ_LOOP: DO

            READ (data_unit,iostat=ios,fmt='(A)') line_read

            line = line_read

! Ignore blank line, get rid of comments, extraneous characters

            IF (ios/=0) THEN

              more_lines = .FALSE.

              RETURN

            END IF

            IF (LEN_TRIM(line)<1) CYCLE READ_LOOP

            CALL end_line(line)

            CALL alphanum(line)

            IF (LEN_TRIM(line)<1) CYCLE READ_LOOP

            CALL low_case_string(line)

            more_lines = .TRUE.

            RETURN

          END DO READ_LOOP

        END SUBROUTINE get_line

!.....................................................................

        SUBROUTINE get_n_integers(line,n_wanted,numbers_obtained)
!!! Gets n integers starting at wherever qlex left off parsing line
! .. Scalar Arguments ..
          INTEGER, INTENT (IN) :: n_wanted
          CHARACTER (max_string), INTENT (OUT) :: line
! ..
! .. Array Arguments ..
          INTEGER, INTENT (OUT) :: numbers_obtained(:)
! ..
! .. Local Scalars ..
          INTEGER :: n_obtained
          LOGICAL :: more_lines
! ..
          n_obtained = 0

          DO

            is_more = qlex(line,qstart,curtyp,cval,lcval,ival,rval,dval, &
              indov)

            IF ( .NOT. is_more) THEN
              CALL get_line(data_unit,line,more_lines)
              IF ( .NOT. more_lines) CALL bat_error_out(200)
              qstart = .TRUE.
              CYCLE
            END IF

            IF (curtyp/='IN') CALL bat_error_out(101)

            n_obtained = n_obtained + 1

            numbers_obtained(n_obtained) = ival

            IF (n_obtained>=n_wanted) EXIT

          END DO

          is_more = qlex(line,qstart,curtyp,cval,lcval,ival,rval,dval, &
            indov)

          IF (is_more) CALL bat_error_out(102)

          RETURN

        END SUBROUTINE get_n_integers

!.....................................................................

        SUBROUTINE process_allelesizes

          CALL get_n_integers(line,n_allele_in,size_in)

          RETURN

        END SUBROUTINE process_allelesizes

!.....................................................................

        SUBROUTINE process_nallele

          CALL get_n_integers(line,1,numbers_obtained)

          n_allele_in = numbers_obtained(1)

          RETURN

        END SUBROUTINE process_nallele

!.....................................................................

        SUBROUTINE process_nrun

          CALL get_n_integers(line,1,numbers_obtained)

          n_dna_in = numbers_obtained(1)

          RETURN

        END SUBROUTINE process_nrun

!.....................................................................

        SUBROUTINE process_nwell

          CALL get_n_integers(line,n_dna_in,numbers_obtained)

          n_well_in(1:n_dna_in) = numbers_obtained(1:n_dna_in)

          RETURN

        END SUBROUTINE process_nwell

!.....................................................................

        SUBROUTINE process_progenitor
! .. Local Scalars ..
          LOGICAL :: progenitor_error
! ..
          CALL get_n_integers(line,2,progenitors_in)

          CALL check_progenitor(progenitors_in,size_in,progenitor_error)

          IF (progenitor_error) CALL bat_error_out(errno=8)

          RETURN

        END SUBROUTINE process_progenitor

!.....................................................................

        SUBROUTINE process_run
! .. Local Scalars ..
          LOGICAL :: type_ok
! ..
!!! READ dna amount for run
          is_more = qlex(line,qstart,curtyp,cval,lcval,ival,rval,dval, &
            indov)

          IF ( .NOT. is_more) CALL bat_error_out(70)

          type_ok = curtyp == 'IN' .OR. curtyp == 'RL'

          IF (type_ok) THEN
            dna_in(n_run_in) = dval
          ELSE
            CALL bat_error_out(71)
          END IF

          CALL get_n_integers(line,n_allele_in,seen_in(n_run_in,:))

          RETURN

        END SUBROUTINE process_run

!.....................................................................

      END SUBROUTINE read_one_bat

!*********************************************************************

    END MODULE problem_in_mod
