    MODULE resize_mod
! .. Implicit None Statement ..
      IMPLICIT NONE
! ..
! .. Parameters ..
      INTEGER, PARAMETER, PRIVATE :: dpkind = KIND(0.0D0)
! ..
! .. Intrinsic Functions ..
      INTRINSIC KIND
! ..
! .. Generic Interface Blocks ..
      INTERFACE size_matrix
        MODULE PROCEDURE double_size_matrix, real_size_matrix, &
          integer_size_matrix
      END INTERFACE
      INTERFACE size_vector
        MODULE PROCEDURE double_size_vector, real_size_vector, &
          integer_size_vector
      END INTERFACE
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE double_size_matrix(matrix_ptr,nrow,ncol)
!!! Resizes a matrix pointed to be matrix_ptr to size
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: ncol, nrow
! ..
! .. Array Arguments ..
        REAL (dpkind), POINTER :: matrix_ptr(:,:)
! ..
! .. Local Scalars ..
        LOGICAL :: ok_condition
! ..
! .. Intrinsic Functions ..
        INTRINSIC ASSOCIATED, SIZE
! ..
! .. Executable Statements ..

        ok_condition = ASSOCIATED(matrix_ptr)

        IF (ok_condition) THEN

          ok_condition = (SIZE(matrix_ptr,dim=1)==nrow) .AND. &
            (SIZE(matrix_ptr,dim=2)==ncol)

          IF (ok_condition) THEN

            RETURN

          ELSE

            DEALLOCATE (matrix_ptr)

          END IF

        END IF

        ALLOCATE (matrix_ptr(nrow,ncol))


        RETURN

      END SUBROUTINE double_size_matrix

!*********************************************************************

      SUBROUTINE double_size_vector(vector_ptr,length)
!!! Resizes a vector pointed to be vector_ptr to size length
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: length
! ..
! .. Array Arguments ..
        REAL (dpkind), POINTER :: vector_ptr(:)
! ..
! .. Local Scalars ..
        LOGICAL :: ok_condition
! ..
! .. Intrinsic Functions ..
        INTRINSIC ASSOCIATED, SIZE
! ..
! .. Executable Statements ..

        ok_condition = ASSOCIATED(vector_ptr)

        IF (ok_condition) THEN

          ok_condition = (SIZE(vector_ptr)==length)

          IF (ok_condition) THEN

            RETURN

          ELSE

            DEALLOCATE (vector_ptr)

          END IF

        END IF

!!! Here vector pointer is deallocated

        ALLOCATE (vector_ptr(length))

        RETURN

      END SUBROUTINE double_size_vector

!*********************************************************************

      SUBROUTINE integer_size_matrix(matrix_ptr,nrow,ncol)
!!! Resizes a matrix pointed to be matrix_ptr to size
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: ncol, nrow
! ..
! .. Array Arguments ..
        INTEGER, POINTER :: matrix_ptr(:,:)
! ..
! .. Local Scalars ..
        LOGICAL :: ok_condition
! ..
! .. Intrinsic Functions ..
        INTRINSIC ASSOCIATED, SIZE
! ..
! .. Executable Statements ..

        ok_condition = ASSOCIATED(matrix_ptr)

        IF (ok_condition) THEN

          ok_condition = (SIZE(matrix_ptr,dim=1)==nrow) .AND. &
            (SIZE(matrix_ptr,dim=2)==ncol)

          IF (ok_condition) THEN

            RETURN

          ELSE

            DEALLOCATE (matrix_ptr)

          END IF

        END IF

        ALLOCATE (matrix_ptr(nrow,ncol))


        RETURN

      END SUBROUTINE integer_size_matrix

!*********************************************************************

      SUBROUTINE integer_size_vector(vector_ptr,length)
!!! Resizes a vector pointed to be vector_ptr to size length
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: length
! ..
! .. Array Arguments ..
        INTEGER, POINTER :: vector_ptr(:)
! ..
! .. Local Scalars ..
        LOGICAL :: ok_condition
! ..
! .. Intrinsic Functions ..
        INTRINSIC ASSOCIATED, SIZE
! ..
! .. Executable Statements ..

        ok_condition = ASSOCIATED(vector_ptr)

        IF (ok_condition) THEN

          ok_condition = (SIZE(vector_ptr)==length)

          IF (ok_condition) THEN

            RETURN

          ELSE

            DEALLOCATE (vector_ptr)

          END IF

        END IF

!!! Here vector pointer is deallocated

        ALLOCATE (vector_ptr(length))

        RETURN

      END SUBROUTINE integer_size_vector

!*********************************************************************

      SUBROUTINE real_size_matrix(matrix_ptr,nrow,ncol)
!!! Resizes a matrix pointed to be matrix_ptr to size
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: ncol, nrow
! ..
! .. Array Arguments ..
        REAL, POINTER :: matrix_ptr(:,:)
! ..
! .. Local Scalars ..
        LOGICAL :: ok_condition
! ..
! .. Intrinsic Functions ..
        INTRINSIC ASSOCIATED, SIZE
! ..
! .. Executable Statements ..

        ok_condition = ASSOCIATED(matrix_ptr)

        IF (ok_condition) THEN

          ok_condition = (SIZE(matrix_ptr,dim=1)==nrow) .AND. &
            (SIZE(matrix_ptr,dim=2)==ncol)

          IF (ok_condition) THEN

            RETURN

          ELSE

            DEALLOCATE (matrix_ptr)

          END IF

        END IF

        ALLOCATE (matrix_ptr(nrow,ncol))


        RETURN

      END SUBROUTINE real_size_matrix

!*********************************************************************

      SUBROUTINE real_size_vector(vector_ptr,length)
!!! Resizes a vector pointed to be vector_ptr to size length
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (IN) :: length
! ..
! .. Array Arguments ..
        REAL, POINTER :: vector_ptr(:)
! ..
! .. Local Scalars ..
        LOGICAL :: ok_condition
! ..
! .. Intrinsic Functions ..
        INTRINSIC ASSOCIATED, SIZE
! ..
! .. Executable Statements ..

        ok_condition = ASSOCIATED(vector_ptr)

        IF (ok_condition) THEN

          ok_condition = (SIZE(vector_ptr)==length)

          IF (ok_condition) THEN

            RETURN

          ELSE

            DEALLOCATE (vector_ptr)

          END IF

        END IF

!!! Here vector pointer is deallocated

        ALLOCATE (vector_ptr(length))

        RETURN

      END SUBROUTINE real_size_vector

!*********************************************************************

    END MODULE resize_mod
