    PROGRAM sppcr
! .. Use Statements ..
      USE get_numbers_mod
      USE one_data_set_mod
      USE open_file_mod
      USE print_it
      USE problem_in_mod
      USE results_out_mod
      USE structures_mod
! ..
! .. Local Scalars ..
      INTEGER :: action, data_unit, rep_unit, sim_unit
      LOGICAL :: data_generated, from_truth, write_simulations
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: max_file_len = 256
! ..
! .. Dependents ..
      CHARACTER (max_file_len) :: data_file_name
! ..
      CALL banner

      DO

        data_generated = .FALSE.

        num_subs = 0

        message_format = '(/5X,''     What do you want to do?''//5X,'' &
          &    (0) Exit this program''//5X,''     (1) Read &
          &data extracted from a filemaker file.''/5X,''   &
          &      (See the documentation for the format)''//5X,'' &
          &    (2) Read data from a ''''batch'''' input file''/5X,'' &
          &        (See the documentation for the format)''//5X,'' &
          &    (3) Interactively enter a data set.''//5X,'' &
          &    (4) Generate random data from parameters that &
          &you specify.''/)'

        CALL get_numbers(format=message_format,lo=0,hi=4,x=action)

        from_truth = .FALSE.

        rep_unit = -1

        sim_unit = -1

        SELECT CASE (action)

        CASE (0) ! Stop

          STOP 'User specified program exit'

        CASE (1) ! Read one data set MJS format

          message_format = '(/5X,''Enter the  name of  the &
            & file  containing the data  in   the''/5X,''standard &
            &format for this analysis.''/)'

          CALL open_file(mssg=message_format,name=data_file_name, &
            unit=data_unit,read=.TRUE.)

          CALL read_mjs_one(data_unit)

          CALL one_data_set(data_file_name)

        CASE (2) ! Interactively enter one data set

          num_subs = 0

          message_format = '(/5X,''Enter the name of the file &
            &containing the data in ''''batch'''' format.''/)'

          CALL open_file(mssg=message_format,name=data_file_name, &
            unit=data_unit,read=.TRUE.)

          CALL read_one_bat(data_unit)

!           WRITE (*,*) 'After reading one batch file'

!           CALL open_output_files(data_file_name,rep_unit,sim_unit)

!           CALL print_data(rep_unit)

          CALL one_data_set(data_file_name)

        CASE (3) ! Interactively enter one data set

          CALL interactive_read

          data_file_name = 'interactive.data'

          CALL one_data_set(data_file_name)

        CASE (4) ! Randomly generate a data set

          data_generated = .TRUE.

          CALL generate_parameters_in(from_truth,write_simulations, &
            rep_unit)

          data_file_name = 'generated.data'

          CALL one_data_set(data_file_name)

        END SELECT

      END DO

    CONTAINS

!*********************************************************************

      SUBROUTINE banner
! .. Use Statements ..
        USE print_it
! ..
! .. Parameters ..
        CHARACTER (*), PARAMETER :: banner_format = '(//5X,'' &
          &                               SPPCR''/5X,''    &
          &                  Version 1.0: January 2003''//5X,'' &
          &                 Estimates Allele Frequencies from''/5X,'' &
          &                     Small Pool PCR Experiments''//5X,'' &
          &                           Barry W. Brown''//5X,''This &
          &work supported by grants CA95567 and CA16672 from &
          &the National''/5X,''Cancer Institute.''//5X,''Copyright &
          &2003 for:''/5X,''               The University of &
          &Texas, M.D. Anderson Cancer Center''/5X,''      &
          &         Department of Biomathematics, Box 237''/5X,'' &
          &              1515 Holcombe Boulevard''/5X,''   &
          &            Houston, TX      77030''/5X,''Contact: &
          &BWB at above address or bwb@mdanderson.org''//5X,''This &
          &program may be freely  copied and distributed; it &
          &can be obtained''/5X,''from''/5X,''             &
          &  http://odin.mdacc.tmc.edu/anonftp/'')'
! ..
        WRITE (*,banner_format)

        CALL hold

        RETURN

      END SUBROUTINE banner

!*********************************************************************

      SUBROUTINE one_data_set(data_file_name)
! .. Scalar Arguments ..
        CHARACTER (max_file_len), INTENT (IN) :: data_file_name
! ..
        CALL open_output_files(data_file_name,rep_unit,sim_unit)

        CALL fit_one(bootstrap=.FALSE.,write_simulations=.FALSE., &
          sim_unit=-1)

        CALL bootstrap(from_truth=from_truth,write_simulations= &
          write_simulations,sim_unit=sim_unit)


        IF (data_generated) CALL print_generate_params(rep_unit)

        CALL print_data(rep_unit)

        CALL print_mu(rep_unit)

        CALL print_answers(rep_unit)

      END SUBROUTINE one_data_set

!*********************************************************************

      SUBROUTINE open_output_files(data_file_name,rep_unit,sim_unit)
!!! Open report file and if needed simulation output file
! .. Use Statements ..
        USE file_mod
        USE open_file_mod
        USE print_it
        USE results_out_mod
! ..
! .. Scalar Arguments ..
        INTEGER, INTENT (OUT) :: rep_unit, sim_unit
        CHARACTER (max_file_len), INTENT (IN) :: data_file_name
! ..
! .. Local Scalars ..
        CHARACTER (max_file_len) :: ext, name, path, rep_file_name, &
          sim_file_name
! ..
! .. Intrinsic Functions ..
        INTRINSIC LEN_TRIM, TRIM
! ..
        WRITE (*,90000) TRIM(data_file_name)
90000   FORMAT (' Processing input file with the following name:'/5X,A)

        CALL parse_file_name(data_file_name,path,name,ext)

        IF (LEN_TRIM(name)==0) THEN
          WRITE (*,*) 'Blank file name not allowed -- ABORT'
          STOP 'Blank file name not allowed -- ABORT'
        ELSE

!!! Open report (answer) file

          rep_file_name = TRIM(name) // '.ans'

          CALL open_named_file(file_name=rep_file_name,unit=rep_unit, &
            read=.FALSE.)
          IF (rep_unit>0) THEN
            WRITE (*,90100) TRIM(rep_file_name)
90100       FORMAT ( &
              ' Results will be written to the screen and to file: '/5X, &
              A)
          ELSE
            WRITE (*,90200)
90200       FORMAT (5X,'The answer file could not be opened &
              &for writing.  Answers will not be'/5X, &
              'written to a file but only to the screen.')
          END IF
        END IF


!!! If requested open simulation out file

        IF (write_simulations) THEN
          sim_file_name = TRIM(name) // '.sim'
          CALL open_named_file(file_name=sim_file_name,unit=sim_unit, &
            read=.FALSE.)
          IF (sim_unit>0) THEN
            WRITE (*,90300) TRIM(sim_file_name)
90300       FORMAT (' Simulation results will be written to ', &
              'the file: '/5X,A)
          ELSE
            WRITE (*,90400)
90400       FORMAT (5X,'The file to which simulation results &
              &are to be written cannot be opened'/5X,'for &
              &writing.  Hence the simulation results will &
              &not be written out.')
          END IF

        END IF

        CALL hold

      END SUBROUTINE open_output_files

!*********************************************************************

    END PROGRAM sppcr
