    MODULE zero_finder
!!!**********************************************************************
!!!   These variables provide the  status of the zero_finder  routine either
!!!   currently or most recently in use.
!!!
!!!   zf_current_status:
!!!      1 - A reverse communication zero-finder is in use
!!!      0 - The most recently used zero-finder completed successfully
!!!     -1 - The most recently used zero-finder completed unsuccessfully
!!!
!!!   The two logical status variables are defined only if zf_current_status
!!!   == -1.
!!!
!!!   zf_crash_left: .TRUE. iff the zero finder tried to exceed the leftmost
!!!   (smallest) allowable value of x
!!!                  .FALSE. iff the zero finder tried to exceed the rightmost
!!!   (largest) allowable value of x
!!!
!!!
!!!   zf_crash_hi: .TRUE. iff f(x) > 0 at the boundary of allowable values
!!!                of x
!!!                .FALSE. iff f(x) > 0 at the boundary of allowable values
!!!                of x
!!!**********************************************************************
!!!                    Default Accuracy Parameters
!!!                    User Set Accuracy Parameters
!!!     A Flag to Tell Whether the User has Set the Parameters
! .. Implicit None Statement ..
      IMPLICIT NONE
! ..
! .. Default Accessibility ..
      PRIVATE
! ..
! .. Parameters ..
      INTEGER, PARAMETER :: common_tolerances_are_set = -123456
      INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
      INTEGER, PARAMETER :: interval_neps = 100
      REAL (dpkind), PARAMETER :: converge = 1.0E-6_dpkind
      REAL (dpkind), PARAMETER :: default_abs_tol = converge
      REAL (dpkind), PARAMETER :: default_rel_step = 1.0E0_dpkind
      REAL (dpkind), PARAMETER :: default_rel_tol = converge
      REAL (dpkind), PARAMETER :: default_step_multiplier = 2.0_dpkind
      REAL (dpkind), PARAMETER :: infinity = 1.0E35_dpkind
      REAL (dpkind), PARAMETER :: interval_init_f = 1.0E5_dpkind
      REAL (dpkind), PARAMETER :: interval_lambda = 0.7_dpkind
      REAL (dpkind), PARAMETER :: interval_mu = 0.5_dpkind
      REAL (dpkind), PARAMETER :: small_step = 1.0E-4_dpkind
      REAL (dpkind), PARAMETER :: default_abs_step = small_step
      REAL (dpkind), PARAMETER :: default_hi_limit = infinity
      REAL (dpkind), PARAMETER :: default_low_limit = -infinity
! ..
! .. Derived Type Declarations ..
      TYPE, PUBLIC :: zf_locals
        PRIVATE
        REAL (dpkind) :: step_absstp
        REAL (dpkind) :: common_abstol
        REAL (dpkind) :: common_big
        REAL (dpkind) :: step_fbig
        REAL (dpkind) :: step_fsmall
        REAL (dpkind) :: step_relstp
        REAL (dpkind) :: common_reltol
        REAL (dpkind) :: common_small
        REAL (dpkind) :: step_step
        REAL (dpkind) :: step_stpmul
        REAL (dpkind) :: step_xhi
        REAL (dpkind) :: step_xlb
        REAL (dpkind) :: step_xlo
        REAL (dpkind) :: step_xsave
        REAL (dpkind) :: step_xub
        REAL (dpkind) :: step_yy
        INTEGER :: step_from
        INTEGER :: common_tolerances_set = 0
        LOGICAL :: step_qbdd
        LOGICAL :: step_qcond
        LOGICAL :: step_qincr
        LOGICAL :: step_qlim
        LOGICAL :: step_qok
        LOGICAL :: step_qup
        LOGICAL :: common_called_from_step
        REAL (dpkind) :: interval_a
        REAL (dpkind) :: interval_a0
        REAL (dpkind) :: interval_b
        REAL (dpkind) :: interval_b0
        REAL (dpkind) :: interval_c
        REAL (dpkind) :: interval_d
        REAL (dpkind) :: interval_e
        REAL (dpkind) :: interval_u
        REAL (dpkind) :: interval_fa
        REAL (dpkind) :: interval_fb
        REAL (dpkind) :: interval_fc
        REAL (dpkind) :: interval_fd
        REAL (dpkind) :: interval_fe
        REAL (dpkind) :: interval_fu
        REAL (dpkind) :: interval_tol0
        REAL (dpkind) :: interval_tol
        REAL (dpkind) :: interval_eps
        REAL (dpkind) :: interval_lenab
        INTEGER :: interval_from
        INTEGER :: interval_itnum
        INTEGER :: common_current_status
        LOGICAL :: common_crash_hi
        LOGICAL :: common_crash_left
        REAL (dpkind) :: common_zf_bound_low
        REAL (dpkind) :: common_zf_bound_high
      END TYPE zf_locals
! ..
! .. Local Scalars ..
      REAL (dpkind), PUBLIC :: zf_bound_high, zf_bound_low
      INTEGER, PUBLIC :: zf_current_status = 1
      LOGICAL, PUBLIC :: zf_crash_hi, zf_crash_left
! ..
! .. Intrinsic Functions ..
      INTRINSIC KIND
! ..
! .. Public Statements ..
      PUBLIC :: final_zf_state, interval_zf, rc_interval_zf, rc_step_zf, &
        set_zero_finder, step_zf
! ..
! .. Dependents ..
      TYPE (zf_locals), SAVE :: default_locals_zf
! ..
    CONTAINS

!*********************************************************************

      SUBROUTINE final_zf_state(local,status,crash_left,crash_hi, &
          left_end,right_end)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Structure Arguments ..
        TYPE (zf_locals) :: local
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (OUT) :: left_end, right_end
        INTEGER, INTENT (OUT) :: status
        LOGICAL, INTENT (OUT) :: crash_hi, crash_left
! ..
! .. Executable Statements ..

        IF (common_current_status==1) THEN
          WRITE (*,*) 'final_zf_state called with status == 1'
          WRITE (*,*) 'This program is aborting'
          STOP 'final_zf_state called with status == 1'
        END IF

        status = common_current_status
        crash_left = common_crash_left
        crash_hi = common_crash_hi

        left_end = common_small
        right_end = common_big

        RETURN

      END SUBROUTINE final_zf_state

!*********************************************************************

      SUBROUTINE interval_zf(f,y,answer,status,local)

        IMPLICIT NONE

        TYPE (zf_locals), OPTIONAL :: local

        INTERFACE
          FUNCTION f(x)
            INTEGER, PARAMETER :: dpkind = KIND(0.0D0)

            REAL (dpkind), INTENT (IN) :: x

            INTRINSIC KIND

        REAL (dpkind) :: f
        END FUNCTION f
        END INTERFACE
        REAL (dpkind), INTENT (OUT) :: answer
        REAL (dpkind), INTENT (IN) :: y
        INTEGER, INTENT (OUT) :: status
        REAL (dpkind) :: fx, x
        LOGICAL :: locals_present
        INTRINSIC PRESENT

        locals_present = PRESENT(local)

        status = 0

        DO
          IF (locals_present) THEN
            CALL rc_interval_local_zf(status,x,fx,local)
          ELSE
            CALL rc_interval_local_zf(status,x,fx,default_locals_zf)
          END IF

          IF (status/=1) EXIT

          fx = f(x) - y

        END DO

        IF (status==0) answer = x

        RETURN

      END SUBROUTINE interval_zf

!*********************************************************************

      SUBROUTINE rc_interval_local_zf(status,x,fx,local)
! DMS This is the routine containing the actual Algorithm
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Intrinsic Functions ..
        INTRINSIC ABS, EPSILON
! ..
! .. Parameters ..
        REAL (dpkind), PARAMETER :: half = 0.5_dpkind
        REAL (dpkind), PARAMETER :: one = 1.0_dpkind
        REAL (dpkind), PARAMETER :: two = 2.0_dpkind
        REAL (dpkind), PARAMETER :: zero = 0.0_dpkind
! ..
! .. Structure Arguments ..
        TYPE (zf_locals) :: local
! ..
! .. Scalar Arguments ..
! .. Local Arguments ..
        REAL (dpkind), INTENT (IN) :: fx
        REAL (dpkind), INTENT (OUT) :: x
        INTEGER, INTENT (INOUT) :: status
! ..
! .. Local Scalars ..
        INTEGER :: iprof
! ..
! .. Executable Statements ..

!---------------------------------------------------------------------
! Finds either an EXACT solution or an APPROXIMATE solution of the
! equation f(x)=0 in the interval [a,b].
! At the begining of each iteration, the current enclosing interval is
! recorded as [a0,b0].
! The FIRST iteration is simply a secant step.
! Starting with the SECOND iteration, THREE STEPS are taken in EACH
! iteration.  The first two steps are either QUADRATIC interpolation
! or CUBIC inverse interpolation. The third step is a double-size
! secant step.  If the diameter of the enclosing interval obtained
! after those three steps is larger than 0.5*(b0-a0), then
! an additional bisection step will be taken.
!  A,B   <-> INPUT:  the initial interval
!            OUTPUT: the enclosing interval at the termination;
!  ROOT  <-- Solution of the equation.
! DMS Each time f(x) is needed, the routine sets status=1 and
!     returns to the caller, then comes back and continues where it
!     left off.
! The algorithm is described in the article:
! Algorithm 748: Enclosing Zeros of Continuous Functions,
! by G. E. Alefeld, F. A. Potra, YiXun Shi,
! ACM Transactions on Mathematical Software, Vol. 21, No. 3, Sep. 1995
! pages 327-344
        IF (status==0) THEN
!!!     Make sure that local has been set
          IF (common_tolerances_set/=common_tolerances_are_set) &
              THEN
            STOP 'Must call set_zero_finder before a zero finding &
              &routine -- ABORT'
          END IF

! Some initializations are done

          IF (common_called_from_step) THEN
            interval_a = step_xlb
            interval_b = step_xub
          ELSE
            interval_a = common_small
            interval_b = common_big
          END IF

! ***********************
          interval_tol = zero
! ***********************

          interval_itnum = 0
          interval_e = interval_init_f
          interval_fe = interval_init_f
          interval_eps = EPSILON(one)

          IF (interval_neps>300) THEN
            interval_tol0 = zero
          ELSE
            interval_tol0 = one/(10.0E0_dpkind**interval_neps)
          END IF


! Go to the caller and compute f(a)

          x = interval_a
          status = 1
          interval_from = 1
          RETURN
        END IF

        IF (status==1) THEN
          SELECT CASE (interval_from)
          CASE (1)
! Called only once to compute: fa = f(a)
            GO TO 10

          CASE (2)
! Called only once to compute: fb = f(b)
            GO TO 20

          CASE (3)
! Called only for ITNUM=1 to compute: fc = f(c)
            GO TO 40

          CASE (4)
            GO TO 60

          CASE (5)
            GO TO 70

          CASE (6)
            GO TO 80

          CASE (7)
            GO TO 90

          END SELECT
        END IF

        IF (status/=0) THEN
          WRITE (*,*) 'Value is: ', status
          STOP 'Illegal input value of status in routine rc_interval_local&
            &_zf'
        END IF

10      CONTINUE
        interval_fa = fx

! Go to the caller and compute f(b)

        x = interval_b
        status = 1
        interval_from = 2
        RETURN

20      CONTINUE
        interval_fb = fx

! Check whether there is a root in [a, b]

        IF ((interval_fa<zero) .AND. (interval_fb<zero)) THEN
          status = -1
          zf_current_status = -1
          zf_crash_left = interval_fa < interval_fb
          zf_crash_hi = .FALSE.
          RETURN
        ELSE IF ((interval_fa>zero) .AND. (interval_fb>zero)) &
            THEN
          status = -1
          zf_current_status = -1
          zf_crash_left = interval_fa > interval_fb
          zf_crash_hi = .TRUE.
          RETURN
        END IF

! Iteration starts.
! The enclosing interval BEFORE executing the iteration is recorded
! as [a0, b0].

30      interval_a0 = interval_a
        interval_b0 = interval_b

!     IF (interval_itnum > 2) STOP

! Length of the interval [a, b]
        interval_lenab = interval_b - interval_a

        interval_itnum = interval_itnum + 1

! Calculates the TERMINATION criterion.
! STOPS the procedure if the criterion is satisfied.

        CALL ftol

        IF (interval_lenab<=interval_tol) THEN
          GO TO 100
        END IF

! For the FIRST iteration, SECANT STEP is taken.

!!! ORIGINAL -- Changed to go to avoid jumping into IF structure
!!!  IF (interval_itnum == 1) THEN

        IF (interval_itnum/=1) GO TO 50
        interval_c = interval_a - (interval_fa/( &
          interval_fb-interval_fa))*interval_lenab

! Call subroutine "BRACKT" to get a shrinked enclosing interval
! as well as to update the termination criterion.
! STOP the procedure if the criterion is satisfied OR
! the EXACT SOLUTION is obtained.

        CALL adjust_c

        x = interval_c
        interval_from = 3
        status = 1
        RETURN

40      CONTINUE
        interval_fc = fx

        CALL bracket

        IF (has_converged()) THEN
          GO TO 100
        ELSE
          GO TO 30
        END IF

50      CONTINUE

!!! ORIGINAL END IF
!!!      END IF

! Starting with the SECOND iteration, in the FIRST TWO STEPS, either
! QUADRATIC INTERPOLATION is used by calling the subroutine "Newton_Quadratic"
! or THE CUBIC INVERSE INTERPOLATION is used by calling the subroutine
! "PZERO".
! In the following, if "IPROF" is not equal to 0, then the
! FOUR function values "fa", "fb", "fd", and "fe" are DISTINCT,
! and hence "PZERO" will be called.

        IF ((interval_itnum==2) .OR. (ifabde()==0)) THEN
          CALL newton_quadratic(2)
        ELSE
          CALL pzero

          IF (iisign(interval_c-interval_a)* &
              iisign(interval_c-interval_b)>=0) THEN
            CALL newton_quadratic(2)
          END IF
        END IF

        interval_e = interval_d
        interval_fe = interval_fd

        CALL adjust_c

        x = interval_c
        interval_from = 4
        status = 1
        RETURN

60      CONTINUE
        interval_fc = fx

        CALL bracket

        IF (has_converged()) GO TO 100

        IF (ifabde()==0) THEN
          CALL newton_quadratic(3)
        ELSE
          CALL pzero

          iprof = iisign(interval_c-interval_a)* &
            iisign(interval_c-interval_b)

          IF (iprof>=0) THEN
            CALL newton_quadratic(3)
          END IF
        END IF

        CALL adjust_c

        x = interval_c
        interval_from = 5
        status = 1
        RETURN

70      CONTINUE
        interval_fc = fx

        CALL bracket

        IF (has_converged()) GO TO 100

        interval_e = interval_d
        interval_fe = interval_fd

! Takes the DOUBLE-SIZE SECANT STEP.

        IF (ABS(interval_fa)<ABS(interval_fb)) THEN
          interval_u = interval_a
          interval_fu = interval_fa
        ELSE
          interval_u = interval_b
          interval_fu = interval_fb
        END IF

        interval_c = interval_u - two*(interval_fu/( &
          interval_fb-interval_fa))*interval_lenab

        IF (ABS(interval_c-interval_u)> &
            half*interval_lenab) THEN
          interval_c = half*(interval_a+interval_b)
        END IF

        CALL adjust_c

        x = interval_c
        interval_from = 6
        status = 1
        RETURN

80      CONTINUE
        interval_fc = fx

        CALL bracket

        IF (has_converged()) GO TO 100

! Determines whether an additional BISECTION STEP is needed.
! Takes it if necessary.

        IF (interval_lenab<interval_mu*(interval_b0- &
            interval_a0)) THEN
          GO TO 30
        END IF

        interval_e = interval_d
        interval_fe = interval_fd

        interval_c = half*(interval_a+interval_b)

        CALL adjust_c

        x = interval_c
        interval_from = 7
        status = 1
        RETURN

90      CONTINUE
        interval_fc = fx

        CALL bracket

        IF (has_converged()) THEN
          GO TO 100
        ELSE
          GO TO 30
        END IF

! Terminates the procedure and return the "root" in x

100     CONTINUE

        status = 0
        zf_current_status = 0
        zf_bound_low = interval_a
        zf_bound_high = interval_b
        common_zf_bound_low = zf_bound_low
        common_zf_bound_high = zf_bound_high
        x = half*(zf_bound_low+zf_bound_high)

        RETURN


!-----------------------------------------------------------------------

      CONTAINS

!.....................................................................

        SUBROUTINE adjust_c
          IMPLICIT NONE
          interval_tol = interval_lambda*interval_tol

          IF ((interval_lenab)<=two*interval_tol) THEN
            interval_c = half*(interval_a+interval_b)
          ELSE IF (interval_c<=interval_a+interval_tol) THEN
            interval_c = interval_a + interval_tol
          ELSE
            IF (interval_c>=interval_b-interval_tol) THEN
              interval_c = interval_b - interval_tol
            END IF
          END IF

        END SUBROUTINE adjust_c

!.....................................................................

        SUBROUTINE bracket
          IF (interval_fc==zero) THEN
            interval_a = interval_c
            interval_fa = zero
            interval_d = zero
            interval_fd = zero

            RETURN
          END IF

          IF ((iisign(interval_fa)*iisign(interval_fc))<0) &
              THEN
            interval_d = interval_b
            interval_fd = interval_fb
            interval_b = interval_c
            interval_fb = interval_fc
          ELSE
            interval_d = interval_a
            interval_fd = interval_fa
            interval_a = interval_c
            interval_fa = interval_fc
          END IF

          interval_lenab = interval_b - interval_a

          CALL ftol

          RETURN

        END SUBROUTINE bracket

!.....................................................................

        SUBROUTINE ftol

          IF (ABS(interval_fb)<=ABS(interval_fa)) THEN
            interval_tol = two*(interval_tol0+two*ABS( &
              interval_b)*interval_eps)
          ELSE
            interval_tol = two*(interval_tol0+two*ABS( &
              interval_a)*interval_eps)
          END IF

        END SUBROUTINE ftol

!.....................................................................

        FUNCTION has_converged()
          LOGICAL :: has_converged

          has_converged = (interval_fa==zero) .OR. &
            (interval_lenab<=interval_tol)

        END FUNCTION has_converged

!.....................................................................

        FUNCTION ifabde()
! Implicit None Statement ..
! .. Implicit None Statement ..
          IMPLICIT NONE
! ..
! .. Function Return Value ..
          INTEGER :: ifabde
! ..
! .. Executable Statements ..

          ifabde = iisign(interval_fa-interval_fb)* &
            iisign(interval_fa-interval_fd)* &
            iisign(interval_fa-interval_fe)* &
            iisign(interval_fb-interval_fd)* &
            iisign(interval_fb-interval_fe)* &
            iisign(interval_fd-interval_fe)

        END FUNCTION ifabde

!.....................................................................

        FUNCTION iisign(x)
!     Returns the SIGN of variable "x".
! .. Implicit None Statement ..
          IMPLICIT NONE
! ..
! .. Function Return Value ..
          INTEGER :: iisign
! ..
! .. Scalar Arguments ..
          REAL (dpkind), INTENT (IN) :: x
! ..
! .. Executable Statements ..

          IF (x>zero) THEN
            iisign = 1
          ELSE IF (x==zero) THEN
            iisign = 0
          ELSE
            iisign = -1
          END IF

          RETURN
        END FUNCTION iisign

!.....................................................................

        SUBROUTINE newton_quadratic(k)

          INTEGER :: k

          REAL (dpkind) :: a0, a1, a2, pc, pdc
          INTEGER :: i, ierror

          ierror = 0

          a0 = interval_fa
          a1 = (interval_fb-interval_fa)/interval_lenab
          a2 = ((interval_fd-interval_fb)/ &
            (interval_d-interval_b)-a1)/ &
            (interval_d-interval_a)

10        IF (a2==zero .OR. ierror==1) THEN
            interval_c = interval_a - a0/a1

            RETURN
          END IF

          IF (iisign(a2)*iisign(interval_fa)>0) THEN
            interval_c = interval_a
          ELSE
            interval_c = interval_b
          END IF

          DO i = 1, k
            IF (ierror==0) THEN
              pc = a0 + (a1+a2*(interval_c-interval_b))*(interval_c-interval_a)
              pdc = a1 + a2*((two*interval_c)-(interval_a+ &
                interval_b))
              IF (pdc==zero) THEN
                ierror = 1
              ELSE
                interval_c = interval_c - pc/pdc
              END IF
            END IF
          END DO

          IF (ierror==1) GO TO 10

          RETURN


        END SUBROUTINE newton_quadratic

!.....................................................................

        SUBROUTINE pzero
          REAL (dpkind) :: const, d21, d31, d32, q11, q21, q22, q31, q32, &
            q33


          q11 = (interval_d-interval_e) * (interval_fd/(interval_fe-interval_fd))
          q21 = (interval_b-interval_d) * (interval_fb/(interval_fd-interval_fb))

          const = -interval_lenab/(interval_fb-interval_fa)
          q31 = const*interval_fa
          d21 = (interval_b-interval_d)* (interval_fd/(interval_fd-interval_fb))
          d31 = const*interval_fb

          q22 = (d21-q11)*(interval_fb/(interval_fe-interval_fb))
          const = (d31-q21)/(interval_fd-interval_fa)
          q32 = const*interval_fa
          d32 = const*interval_fd
          q33 = (d32-q22)*(interval_fa/(interval_fe-interval_fa))

          interval_c = interval_a + q31 + q32 + q33

          RETURN

        END SUBROUTINE pzero

!.....................................................................

      END SUBROUTINE rc_interval_local_zf

!*********************************************************************

      SUBROUTINE rc_interval_zf(status,x,fx,local)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Structure Arguments ..
        TYPE (zf_locals), OPTIONAL :: local
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (IN) :: fx
        REAL (dpkind), INTENT (OUT) :: x
        INTEGER, INTENT (INOUT) :: status
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        IF (PRESENT(local)) THEN
          CALL rc_interval_local_zf(status,x,fx,local)
        ELSE
          CALL rc_interval_local_zf(status,x,fx,default_locals_zf)
        END IF

        RETURN

      END SUBROUTINE rc_interval_zf

!*********************************************************************

      SUBROUTINE rc_step_local_zf(status,x,fx,local)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Intrinsic Functions ..
        INTRINSIC ABS, MAX, MIN
! ..
! .. Parameters ..
        REAL (dpkind), PARAMETER :: half = 0.5_dpkind
        REAL (dpkind), PARAMETER :: one = 1.0_dpkind
        REAL (dpkind), PARAMETER :: two = 2.0_dpkind
        REAL (dpkind), PARAMETER :: zero = 0.0_dpkind
! ..
! .. Structure Arguments ..
        TYPE (zf_locals) :: local
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (IN) :: fx
        REAL (dpkind), INTENT (OUT) :: x
        INTEGER, INTENT (INOUT) :: status

        IF (status==1) THEN
          SELECT CASE (step_from)
          CASE (1)
            GO TO 10

          CASE (2)
            GO TO 20

          CASE (3)
            GO TO 30

          CASE (4)
            GO TO 50

          CASE (5)
            GO TO 80

          CASE (6)
            GO TO 100
          END SELECT
        END IF

        IF (status/=0) THEN
          WRITE (*,*) 'Value is: ', status
          STOP &
            'Illegal input value of status in routine rc_step_local_zf'
        END IF

        common_current_status = status

        IF ( .NOT. qxmon(common_small,x,common_big)) THEN
          WRITE (*,*) 'Small, x, big not monotone in rc_step_local_zf'
          WRITE (*,*) 'Value of small: ', common_small
          WRITE (*,*) 'Value of x: ', x
          WRITE (*,*) 'Value of big: ', common_big
          STOP ' x, big not monotone in rc_step_local_zf'
        END IF

        step_xsave = x

        x = common_small
        step_from = 1
        status = 1
        common_current_status = status
        RETURN

10      CONTINUE
        step_fsmall = fx
        x = common_big
        step_from = 2
        status = 1
        common_current_status = status
        RETURN

20      CONTINUE

        step_fbig = fx
        step_qincr = step_fbig > step_fsmall
        IF (step_qincr) THEN
          IF ( .NOT. (step_fsmall<=zero)) THEN
            status = -1
            zf_current_status = -1
            zf_crash_left = .TRUE.
            zf_crash_hi = .TRUE.
            common_current_status = status
            common_crash_left = zf_crash_left
            common_crash_hi = zf_crash_hi
            RETURN
          END IF
          IF ( .NOT. (step_fbig>=zero)) THEN
            status = -1
            zf_current_status = -1
            zf_crash_left = .FALSE.
            zf_crash_hi = .FALSE.
            RETURN
          END IF
        ELSE
          IF ( .NOT. (step_fsmall>=zero)) THEN
            status = -1
            zf_current_status = -1
            zf_crash_left = .TRUE.
            zf_crash_hi = .FALSE.
            RETURN
          END IF
          IF ( .NOT. (step_fbig<=zero)) THEN
            status = -1
            zf_current_status = -1
            zf_crash_left = .FALSE.
            zf_crash_hi = .TRUE.
            RETURN
          END IF
        END IF

        x = step_xsave
        step_step = MAX(step_absstp,step_relstp*ABS(x))
        step_from = 3
        status = 1
        RETURN

30      CONTINUE

        step_yy = fx
        IF (step_yy==zero) THEN
          status = 0
          zf_current_status = 0
          step_qok = .TRUE.
          RETURN
        END IF
        step_qup = (step_qincr .AND. (step_yy<zero)) &
          .OR. ( .NOT. step_qincr .AND. (step_yy>zero))

        IF ( .NOT. step_qup) GO TO 60
        step_xlb = step_xsave
        step_xub = MIN(step_xlb+step_step, common_big)
40      CONTINUE
        x = step_xub
        step_from = 4
        status = 1
        RETURN

50      CONTINUE
        step_yy = fx
        step_qbdd = (step_qincr .AND. (step_yy>=zero)) &
          .OR. ( .NOT. step_qincr .AND. (step_yy<=zero))
        step_qlim = step_xub >= common_big
        step_qcond = step_qbdd .OR. step_qlim
        IF ( .NOT. (step_qcond)) THEN
          step_step = step_stpmul*step_step
          step_xlb = step_xub
          step_xub = MIN(step_xlb+step_step, &
            common_big)
          GO TO 40
        END IF
        IF (step_qlim .AND. .NOT. step_qbdd) THEN
          status = -1
          zf_current_status = -1
          zf_crash_left = .FALSE.
          zf_crash_hi = .NOT. step_qincr
          x = common_big
          RETURN
        END IF
        GO TO 90

60      CONTINUE
        step_xub = step_xsave
        step_xlb = MAX(step_xub-step_step, common_small)

70      CONTINUE
        x = step_xlb
        step_from = 5
        status = 1
        RETURN

80      CONTINUE
        step_yy = fx
        step_qbdd = (step_qincr .AND. (step_yy<=zero)) &
          .OR. ( .NOT. step_qincr .AND. (step_yy>=zero))
        step_qlim = step_xlb <= common_small
        step_qcond = step_qbdd .OR. step_qlim

        IF ( .NOT. (step_qcond)) THEN
          step_step = step_stpmul*step_step
          step_xub = step_xlb
          step_xlb = MAX(step_xub-step_step, &
            common_small)
          GO TO 70
        END IF
        IF (step_qlim .AND. .NOT. step_qbdd) THEN
          status = -1
          zf_current_status = -1
          zf_crash_left = .TRUE.
          zf_crash_hi = step_qincr
          x = common_small
          RETURN
        END IF


90      CONTINUE
        status = 0
        zf_current_status = 0
        common_called_from_step = .TRUE.

100     CALL rc_interval_local_zf(status,x,fx,local)

        step_from = 6

        IF (status/=1) status = 0
        RETURN


      CONTAINS

!.....................................................................

        FUNCTION qxmon(zx,zy,zz)
! .. Implicit None Statement ..
          IMPLICIT NONE
! ..
! .. Function Return Value ..
          LOGICAL :: qxmon
! ..
! .. Scalar Arguments ..
          REAL (dpkind), INTENT (IN) :: zx, zy, zz
! ..
! .. Executable Statements ..

          qxmon = zx <= zy .AND. zy <= zz
        END FUNCTION qxmon

!.....................................................................

      END SUBROUTINE rc_step_local_zf

!*********************************************************************

      SUBROUTINE rc_step_zf(status,x,fx,local)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Structure Arguments ..
        TYPE (zf_locals), OPTIONAL :: local
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (IN) :: fx
        REAL (dpkind), INTENT (OUT) :: x
        INTEGER, INTENT (INOUT) :: status
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        IF (PRESENT(local)) THEN
          CALL rc_step_local_zf(status,x,fx,local)
        ELSE
          CALL rc_step_local_zf(status,x,fx,default_locals_zf)
        END IF


        RETURN

      END SUBROUTINE rc_step_zf

!*********************************************************************

      SUBROUTINE set_zero_finder(low_limit,hi_limit,abs_tol,rel_tol, &
          abs_step,rel_step,step_multiplier,local)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Structure Arguments ..
        TYPE (zf_locals), OPTIONAL :: local
! ..
! .. Scalar Arguments ..
        REAL (dpkind), OPTIONAL, INTENT (IN) :: abs_step, abs_tol, &
          hi_limit, low_limit, rel_step, rel_tol, step_multiplier
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Local Scalars ..
        LOGICAL :: locals_present
! ..
! .. Executable Statements ..

!!! Set flag to show that this routine has been called
!!! and that the interval routine was not called from the step routine
!!! If the interval routine was so callled, the flag will be reset
!!! by the step routine
        locals_present = PRESENT(local)

        IF (locals_present) THEN
          common_tolerances_set = common_tolerances_are_set
          common_called_from_step = .FALSE.
        ELSE
          default_locals_zf%common_tolerances_set = &
            common_tolerances_are_set
          default_locals_zf%common_called_from_step = .FALSE.
        END IF

!!!     Low Limit

        IF (PRESENT(low_limit)) THEN
          IF (locals_present) THEN
            common_small = low_limit
          ELSE
            default_locals_zf%common_small = low_limit
          END IF
        ELSE
          IF (locals_present) THEN
            common_small = default_low_limit
          ELSE
            default_locals_zf%common_small = default_low_limit
          END IF
        END IF

!!!     Hi Limit

        IF (PRESENT(hi_limit)) THEN
          IF (locals_present) THEN
            common_big = hi_limit
          ELSE
            default_locals_zf%common_big = hi_limit
          END IF
        ELSE
          IF (locals_present) THEN
            common_big = default_hi_limit
          ELSE
            default_locals_zf%common_big = default_hi_limit
          END IF
        END IF

!!!     Absolute tolerance

        IF (PRESENT(abs_tol)) THEN
          IF (locals_present) THEN
            common_abstol = abs_tol
          ELSE
            default_locals_zf%common_abstol = abs_tol
          END IF
        ELSE
          IF (locals_present) THEN
            common_abstol = default_abs_tol
          ELSE
            default_locals_zf%common_abstol = default_abs_tol
          END IF
        END IF

!     Relative tolerance

        IF (PRESENT(rel_tol)) THEN
          IF (locals_present) THEN
            common_reltol = rel_tol
          ELSE
            default_locals_zf%common_reltol = rel_tol
          END IF
        ELSE
          IF (locals_present) THEN
            common_reltol = default_rel_tol
          ELSE
            default_locals_zf%common_reltol = default_rel_tol
          END IF
        END IF

!     Absolute step size

        IF (PRESENT(abs_step)) THEN
          IF (locals_present) THEN
            step_absstp = abs_step
          ELSE
            default_locals_zf%step_absstp = abs_step
          END IF
        ELSE
          IF (locals_present) THEN
            step_absstp = default_abs_step
          ELSE
            default_locals_zf%step_absstp = default_abs_step
          END IF
        END IF

!     Relative step size

        IF (PRESENT(rel_step)) THEN
          IF (locals_present) THEN
            step_relstp = rel_step
          ELSE
            default_locals_zf%step_relstp = rel_step
          END IF
        ELSE
          IF (locals_present) THEN
            step_relstp = default_rel_step
          ELSE
            default_locals_zf%step_relstp = default_rel_step
          END IF
        END IF

!     Step size multiplier

        IF (PRESENT(step_multiplier)) THEN
          IF (locals_present) THEN
            step_stpmul = step_multiplier
          ELSE
            default_locals_zf%step_stpmul = step_multiplier
          END IF
        ELSE
          IF (locals_present) THEN
            step_stpmul = default_step_multiplier
          ELSE
            default_locals_zf%step_stpmul = default_step_multiplier
          END IF
        END IF

      END SUBROUTINE set_zero_finder

!*********************************************************************

      SUBROUTINE step_zf(f,y,answer,status,local)
! .. Implicit None Statement ..
        IMPLICIT NONE
! ..
! .. Structure Arguments ..
        TYPE (zf_locals), OPTIONAL :: local
! ..
! .. Non-Generic Interface Blocks ..
        INTERFACE
          FUNCTION f(x)
! .. Parameters ..
            INTEGER, PARAMETER :: dpkind = KIND(0.0D0)
! ..
! .. Scalar Arguments ..
            REAL (dpkind), INTENT (IN) :: x
! ..
! .. Intrinsic Functions ..
            INTRINSIC KIND
! ..
! .. Function Return Value ..
            REAL (dpkind) :: f
! ..
          END FUNCTION f
        END INTERFACE
! ..
! .. Scalar Arguments ..
        REAL (dpkind), INTENT (INOUT) :: answer
        REAL (dpkind), INTENT (IN) :: y
        INTEGER, INTENT (OUT) :: status
! ..
! .. Local Scalars ..
        REAL (dpkind) :: fx, x
        LOGICAL :: locals_present
! ..
! .. Intrinsic Functions ..
        INTRINSIC PRESENT
! ..
! .. Executable Statements ..

        locals_present = PRESENT(local)

        status = 0
        x = answer

        DO
          IF (locals_present) THEN
            CALL rc_step_local_zf(status,x,fx,local)
          ELSE
            CALL rc_step_local_zf(status,x,fx,default_locals_zf)
          END IF

          IF (status/=1) EXIT

          fx = f(x) - y

        END DO

        IF (status==0) answer = x

        RETURN

      END SUBROUTINE step_zf

!*********************************************************************

    END MODULE zero_finder
